local addonInfo, addonTable = ...
if not addonTable.KBB then addonTable.KBB = {} end
local KBB = addonTable.KBB
local KBB_Defaults = KBB.defaults

local KBB_CharDB

KBB_Settings = KBB_Settings or {}  -- saved variables
KBB.KBB_Settings_ = {}

KBB.groupcache = {}

KBB.barcache = {}

local KBBDebug = KBB.debug
local KBBLS = KBB.localString
KBB.usedGroupIds = {}
------------------
--
------------------



local function GetBar()
	local bar
	if (#(KBB.barcache) > 0) then
		KBBDebug("getting bar from cache, "..(#(KBB.barcache) - 1).." left")
		bar = table.remove(KBB.barcache)
	else
		KBBDebug("making new bar")
		bar = KBB.KBB_Bar.new()
	end

	return bar
end

local function GetGroup()
	local group
	if(#(KBB.groupcache) > 0) then
		group = table.remove(KBB.groupcache)

	else
		group = KBB.KBB_Group.new()
	end
	return group
end

function KBB.RescanGroup(g)
	KBB.RescanBuffs(g)
	if(g.settings.sorts) then g:sortBars() end
	g:alignBars()
end
function KBB.RescanBuffs(group)
	if (Inspect.Unit.Lookup(group.settings.unit)) then
		KBBDebug("scanning buffs on "..group.settings.unit .. ", in " .. group.settings.id)
		KBBDebug("group is watching "..group.settings.watchType.."s")
		local buffList = Inspect.Buff.List(group.settings.unit)
		KBBDebug("Inspect.Buff.List("..group.settings.unit..") = {")
		if(KBB.debugMode) then
			Library.KLH.tdump(buffList)
		end
		KBBDebug("}")
		if not buffList then KBBDebug("no buffs on "..group.settings.unit) group:removeAllBars() return end
		local buffTable = Inspect.Buff.Detail(group.settings.unit, buffList)
		local matched = {}
		local wtype
		local bar
		-- get new buffs
		local bufCount = 0
		for buffId, buff in pairs(buffTable) do
			bufCount = bufCount + 1
			wtype = buff.debuff and "debuff" or "buff"
			KBBDebug("found "..wtype..": "..buff.name)
			if (group.settings.watchType == wtype and KBB.MatchBuff(buff, group.settings.filters, group.settings.filterType)) then

				KBBDebug("matched " .. buff.name)
				-- find buff's bar
				bar = nil
				bar = group:barWithBuffId(buffId)
				-- add bar if it's not there
				if not bar then
					bar = GetBar()
					bar.frame:SetVisible(true)
					group:addBar(bar)
					bar:setBuffInfo(buffId, buff)
					bar:setBarTextsAndIcon()
					bar:fitText(group.settings.stackMargin)
					bar:update(buff)
				end

				-- add it to list of up-to-moment buffs
				matched[buffId] = true
			end
		end
		KBBDebug("found "..bufCount.. " buffs on "..group.settings.unit)

		-- list of old buffs
		local old = {}
		for _, bar in pairs(group.bars) do
			old[bar.buffInfo.buffId] = true
		end
		-- remove removed buffs' bars
		for b, _ in pairs(old) do
			if (matched[b] ~= true) then
				group:removeBarWithBuffId(b)
			end
		end

		group:updateTimers()
	else
		group:removeAllBars()
	end
end

function KBB.RemoveGroup(group)
	if not group then return nil end
	group:removeAllBars()
	group:loadDefaults()
	local id = tonumber(string.sub(group.settings.id, 9))
	KBB.usedGroupIds[id] = false
	table.insert(KBB.groupcache, group)
	for k, g in ipairs(KBB.groups) do
		if (g.settings.id == group.settings.id) then
			table.remove(KBB.groups, k)
			KBBDebug("removed group frame")
			break
		end
	end
	for k, gS in ipairs(KBB_CharDB.groups) do
		if (gS.id == group.settings.id) then
			table.remove(KBB_CharDB.groups, k)
			KBBDebug("removed group settings")
			break
		end
	end
	group.frame:SetVisible(false)
	return group
end

function KBB.RemoveAllGroups()
	for _, group in pairs(KBB.groups) do
		group.frame:SetVisible(false)
		group:removeAllBars()
		group:loadDefaults()
		local id = tonumber(string.sub(group.settings.id, 9))
		KBB.usedGroupIds[id] = false
		table.insert(KBB.groupcache, group)
	end
	while(next(KBB.groups)) do
		table.remove(KBB.groups, 1)
	end
	while(next(KBB_CharDB.groups)) do
		table.remove(KBB_CharDB.groups, 1)
	end
end

function KBB.RemoveGroupAtIndex(index)
	if index > #(KBB.groups) then return nil end

	local group = table.remove(KBB.groups, index)
	local id = tonumber(string.sub(group.settings.id, 9))
	KBB.usedGroupIds[id] = false

	KBBDebug("removed group frame")
	group:removeAllBars()
	group:loadDefaults()
	table.insert(KBB.groupcache, group)
	for k, g in pairs(KBB_CharDB.groups) do
		if (g.id == group.settings.id) then
			table.remove(KBB_CharDB.groups, k)
			KBBDebug("removed group settings")
			if (KBB.debugMode) then
				Library.KLH.tdump(KBB.KBB_Settings_)
			end
			break
		end
	end
	group.frame:SetVisible(false)
	return group
end

function KBB.AddGroup()
	local group = GetGroup()
	if not group then return nil end     
	
	local id = Library.KLH.getFirstNil(KBB.usedGroupIds)
	group:setId(id)
	KBB.usedGroupIds[id] = true
	group:updateAnchorText()
	
	table.insert(KBB.groups, group)
	if not KBB_CharDB.groups then KBB_CharDB.groups = {} end
	table.insert(KBB_CharDB.groups, group.settings)
	group.frame:SetVisible(true)
	KBB.RescanBuffs(group)

	group:updateFrame()
	group:updateColors()
--	group:updateAnchorText()
	group:alignBars()
--	newGroup:move()

	group.anchor:SetVisible(KBB.showAnchors)
	group.barsContainer:SetVisible(not KBB.showAnchors)
	return group
end

function KBB.GroupWithId(id)
	if not id then return nil end
	for i, g in pairs(KBB.groups) do
		if (g.settings.id == id) then
			return i, g
		end
	end
	return nil
end

------------------
-- anchors
------------------
function KBB.ToggleAnchors()
	KBB.showAnchors = not KBB.showAnchors
	for _, g in pairs(KBB.groups) do
		g.anchor:SetVisible(KBB.showAnchors)
		g.barsContainer:SetVisible(not KBB.showAnchors)
	end
end

local function HighlightAnchor(groupId)
end
------------------

------------------
-- saved variables
------------------
local function UpdateSV()
	local settings = KBB_Settings.settings
	for profile, profileSettings in pairs(settings) do
		if not profileSettings.settings then
			profileSettings.settings = Library.KLH.deepcopy(KBB_Defaults.settings)
		end
		if profileSettings.groups then
			for _, group in pairs(profileSettings.groups) do
				for key, value in pairs(KBB_Defaults.group) do
					if not group[key] then
						group[key] = Library.KLH.deepcopy(value) 
					end
				end
			end
		end
	end
	
	if not KBB_Settings.version or Library.KLH.versionIsOlder(KBB_Settings.version, "2.05") then
		print("updating data format to version 2.05")
		local tt = {"buffColor", "debuffColor", "fontColor"}
		for profile, profileSettings in pairs(settings) do
			if profileSettings.groups then
				for _, group in pairs(profileSettings.groups) do
					for _, key in pairs(tt) do
						if group[key] then
							group[key].r = math.floor(group[key].r * 255)
							group[key].g = math.floor(group[key].g * 255)
							group[key].b = math.floor(group[key].b * 255)
						end	
					end
					if group.backgroundAlpha then 
						group.backgroundAlpha = math.floor (group.backgroundAlpha * 255)
					end
					for _, filter in pairs(group.filters) do
						for _, chunk in ipairs(filter) do
							if chunk.ftype == "n" then
								chunk.value = string.lower(chunk.value)
							end
						end
					end
				end
			end
		end	
		KBB_Settings.version = "2.05"
	end
end
local function LoadSavedVariables()
	
	KBBDebug("loading saved variables")
	KBB_Settings = KBB_Settings or {}
	if not KBB_Settings.settings then KBB_Settings.settings = {} end
	
	local shard = Inspect.Shard().name
	local character = Inspect.Unit.Detail("player").name
	KBB.playerID = Inspect.Unit.Lookup("player")
	local charAtShard = string.format("%s@%s", character, shard)
	if not KBB_Settings.settings[charAtShard] then KBB_Settings.settings[charAtShard] = {} end
		
	UpdateSV()
	-- because adding new things to existing SV is wonky, I'll keep a copy of SV
	-- and just force the data onto 'official' saved variable table on unload
	KBB.KBB_Settings_ = Library.KLH.deepcopy(KBB_Settings) --or {} 	

	KBB.characterDB = KBB.KBB_Settings_.settings[charAtShard]
	KBB_CharDB = KBB.KBB_Settings_.settings[charAtShard]
	if not KBB_CharDB.groups then  KBB_CharDB.groups = {} end
	KBBDebug(#(KBB_CharDB.groups).." groups")
	
	KBB.minimapButton:SetWidth(KBB.characterDB.settings.minimapButtonSize)
	KBB.minimapButton:SetHeight(KBB.characterDB.settings.minimapButtonSize)
	KBB.minimapButton:SetVisible(KBB.characterDB.settings.minimapButton)
	Library.KLH.attachButtonToMinimap(KBB.minimapButton, KBB.characterDB.settings.minimapButtonAngle, KBB.characterDB.settings.minimapButtonOffset)
end

local function LoadGroups()
	KBBDebug("loading groups")
	for _, groupSettings in pairs(KBB_CharDB.groups) do
		KBBDebug("loading "..groupSettings.id)
		local newGroup = GetGroup()
-- 		KBB.usedGroupIds[id] = false
		newGroup.settings = groupSettings
		for key, value in pairs(KBB_Defaults.group) do
			if not newGroup.settings[key] then
				KBBDebug("loading default value for "..key)
				newGroup.settings[key] = Library.KLH.deepcopy(value)
			end
		end
		local id = tonumber(string.sub(newGroup.settings.id, 9))
		if KBB.usedGroupIds[id] then 
			id = Library.KLH.getFirstNil(KBB.usedGroupIds)
			newGroup:setId(id) 
		end
		
		KBB.usedGroupIds[id] = true
		table.insert(KBB.groups, newGroup)
		KBBDebug("loaded group "..newGroup.settings.name)
		newGroup:updateFrame()
		newGroup:updateColors()
		newGroup:updateAnchorText()
		newGroup:alignBars()
		newGroup:move()

		newGroup.frame:SetVisible(true)
	end
end

-- local function Saved(char, shard)
-- 	local charAtShard = string.format("%s@%s", character, shard)
-- 	local settings = KBB.KBB_Settings_.settings
-- 	return settings[charAtShard]
-- end

local loadScanned = false
local function PerformLoadSequence()         -- magic!
	if not loadScanned then 
		if not Inspect.Unit.Detail("player") then
		-- it has no data even though it should.
		-- Hence several events to check - one of them will have the data.
		-- I hope.
			KBBDebug("no player data")
			return
		end
		if not KBB.playerID then KBB.playerID = Inspect.Unit.Lookup("player") end -- inspector may return nil. That's why I check it in several events.
		if (KBB.playerID and (not KBB_CharDB) and Inspect.Unit.Detail("player")) then   -- if player details are available and I need them
			LoadSavedVariables()
		end
		if (KBB_CharDB.groups and not KBB.groups or #(KBB.groups) == 0) then  -- if I loaded svars, but  not created groups  yet
			LoadGroups()
			loadScanned = true
		end
	end
--  	for _, group in pairs(KBB.groups)  do
--  		group:removeAllBars()
-- --   		KBB.RescanGroup(group)
--  	end
end


-- due to *** behaviour of events/inspectors/saved variables
-- I have to check several events in hopes that something has player/saved variables data
-- Magic! 
table.insert(Event.Addon.SavedVariables.Load.End, {
	function (addonId)
		if (addonId == addonInfo.toc.Identifier) then
			KBBDebug("Event.Addon.SavedVariables.Load.End")
 			PerformLoadSequence()
		end
	end,
	addonInfo.toc.Identifier, "loadASLE"})

local function OnUnitAvailable(units)
	KBBDebug("Event.Unit.Available")
	local playerID = Inspect.Unit.Lookup("player")
	if units[playerID] then
		for _, group in pairs(KBB.groups) do
			if group.settings.unit == player then group.unitID = playerID end
			group:removeAllBars()
		end
		if not KBB.loadUA then
			PerformLoadSequence()
			KBB.loadUA = true
		end
	end
end

table.insert(Event.Unit.Available, { OnUnitAvailable, addonInfo.toc.Identifier, "loadUA"})

local function EnforceSavedVariables()
	KBB_Settings = Library.KLH.deepcopy(KBB.KBB_Settings_)
end

table.insert(Event.Addon.Shutdown.Begin, {
	function ()
		EnforceSavedVariables()
	end,
	addonInfo.toc.Identifier, "EnforceSavedVariables"})    
	
function KBB.importGroups(source, dest, rescan)
	local sourceGroups = KBB.KBB_Settings_.settings[source]
	KBB.RemoveAllGroups()
	KBB.KBB_Settings_.settings[dest].groups = Library.KLH.deepcopy(sourceGroups.groups)

	if (rescan) then
		LoadGroups()
		for _, g in pairs(KBB.groups) do
			KBB.RescanGroup(g)
		end
	end
end

function KBB.exportGroups(source, dest)
	if KBB.KBB_Settings_.settings[dest] then
--		print("Profile "..dest.."already exists.")
		return false
	end

	local sourceGroups = KBB.KBB_Settings_.settings[source].groups
	KBB.KBB_Settings_.settings[dest] = {}
	KBB.KBB_Settings_.settings[dest].groups = Library.KLH.deepcopy(sourceGroups)

	return true
end

function KBB.deleteProfile(profile, rescan)
	KBB.KBB_Settings_.settings[profile] = nil

	if rescan then
		LoadGroups()
		for _, g in pairs(KBB.groups) do
			KBB.RescanGroup(g)
		end
	end
end

------------------
-- events
------------------

local function OnBuffAdd(unit, buffs)
	if not KBB.on then return end
	if (KBB.units[Inspect.Unit.Lookup(unit)]) then

		local buffTable = Inspect.Buff.Detail(unit, buffs)
		local wtype
		local bar
		local yOffset
		for k, group in ipairs(KBB.groups) do
-- 			if (Inspect.Unit.Lookup(group.settings.unit) == unit) then
			if (group.unitID == unit) then
				KBBDebug("adding buffs on " .. group.settings.unit..", in "..group.settings.id)
				KBBDebug("group is watching " .. group.settings.watchType .. "s")
				for buffId, buff in pairs(buffTable) do
					KBBDebug("removing duplicate buffs on"  .. group.settings.unit..", in "..group.settings.id)
					group:removeBarWithBuffId(buffId)
					
					wtype = buff.debuff and "debuff" or "buff"
					KBBDebug("found "..wtype..": "..buff.name)
					if (group.settings.watchType == wtype and KBB.MatchBuff(buff, group.settings.filters, group.settings.filterType)) then
						KBBDebug("matched " .. buff.name)
						if  KBB.debugMode then
							print("----------------")
							Library.KLH.tdump(buff)
							print("----------------")
						end
						bar = GetBar()

						yOffset = #(group.bars) * (group.settings.barHeight + group.settings.spacing)
						if group.settings.growth == "up" then yOffset = 0 - yOffset end

						group:addBar(bar)

						bar.frame:SetPoint("TOPLEFT", group.frame, "TOPLEFT", 0, yOffset)
                        

						bar:setBuffInfo(buffId, buff)
						
						if not buff.descriptionComplete then Command.Buff.Describe(unit, buffId) end
						
						bar:setBarTextsAndIcon()
						bar:fitText(group.settings.stackMargin)
						bar:update(buff)
						bar.frame:SetVisible(true)
						
						KBBDebug("solid frame: ", bar.solid:GetBounds())
					end
				end
				KBBDebug("bars in group " .. group.settings.id .. ":")
				for _, bar in ipairs(group.bars) do
					KBBDebug(bar.buffInfo.name, bar.buffInfo.stack or 0, Library.KLH.timeString(bar.buffInfo.duration or 0))
				end

			end
			if(group.settings.sorts) then group:sortBars() group:alignBars() end
		end
	end
end


table.insert(Event.Buff.Add, {function(...) OnBuffAdd(...) end, addonInfo.toc.Identifier, "OnBuffAdd"})

local function OnBuffRemove(unit, buffs)
	if not KBB.on then return end
-- 	KBB.hideBuffTooltip()
	for k, group in pairs(KBB.groups) do
-- 		if (Inspect.Unit.Lookup(group.settings.unit) == unit) then
		if (group.unitID == unit) then
			KBBDebug("removing bar from "..group.settings.id)
			for buffId in pairs(buffs) do
					group:removeBarWithBuffId(buffId)
			end
			group:alignBars()
		end
	end
end

table.insert(Event.Buff.Remove, {function(...) OnBuffRemove(...) end, addonInfo.toc.Identifier, "OnBuffRemove"})

local function OnBuffChange(unit, buffs)
	if not KBB.on then return end
	KBBDebug("----------------")
	KBBDebug("buff change on unit "..unit.." - "..Inspect.Unit.Lookup(unit))
	if (KBB.debugMode) then
		Library.KLH.tdump(buffs)
	end
	KBBDebug("----------------")
	local buffTable = Inspect.Buff.Detail(unit, buffs)
	local watchType
	local bar
	local buffInfo
	for k, group in ipairs(KBB.groups) do
-- 		if (Inspect.Unit.Lookup(group.settings.unit) == unit) then
		if (group.unitID == unit) then
-- 			for buffId, _ in pairs(buffs) do
-- 				buffInfo = Inspect.Buff.Detail(unit, buffId)
			for buffId, buffInfo in pairs(buffTable) do	
				watchType = buffInfo.debuff and "debuff" or "buff"
				if (group.settings.watchType == watchType) then
					for _, bar in pairs(group.bars) do
						if (bar.buffInfo.buffId == buffId) then
							bar:setBuffInfo(buffId, buffInfo)
							bar:setBarTexts()
							bar:fitText(group.settings.stackMargin)
							bar:update(buffInfo)
						end
					end
				end    
			end
			if(group.settings.sorts) then group:sortBars() group:alignBars() end
		end
	end
end

table.insert(Event.Buff.Change, {function(...) OnBuffChange(...) end, addonInfo.toc.Identifier, "OnBuffChange"})


local function OnBuffDescription(unit, buffIds)
	local detail = Inspect.Buff.Detail(unit, buffIds)

	for _, g in ipairs(KBB.groups) do
-- 		if Inspect.Unit.Lookup(g.settings.unit) == unit then
		if g.unitID == unit then
			for _, bar in pairs(g.bars) do
				if detail[bar.buffInfo.buffId] then
					bar.buffInfo.description = detail[bar.buffInfo.buffId].description
				end
			end
		end
	end
end
table.insert(Event.Buff.Description, {function(...) OnBuffDescription(...) end, addonInfo.toc.Identifier, "OnBuffDescription"})


local function OnUnitChange(unit, value)
	if not KBB.on then return end
  	for _, g in pairs(KBB.groups) do
		if g.settings.unit == unit then
			if (value) then
-- 				print(value)
				g.unitID = value
-- 				Command.System.Watchdog.Quiet()
				KBB.RescanGroup(g)
			else
				g.unitID = nil
				g:removeAllBars()
			end
		end
	end
end

for k, v in pairs(KBB.units) do
	table.insert(Library.LibUnitChange.Register(k), {function(value) OnUnitChange(k, value) end, addonInfo.toc.Identifier, "OnUnitChange"})
end

local function OnRoleChange(units, role)
	for unitId in pairs(units) do
		if (unitId == Inspect.Unit.Lookup("player")) then
			for _, g in pairs(KBB.groups) do
				if g.settings.unit == unit then
	    			for _, g in pairs(KBB.groups) do
						KBB.RescanGroup(g)
			    	 end
	 			end
		 	end
		 end
	 end
end

table.insert(Event.Unit.Detail.Role, {function(...) OnRoleChange(...) end, addonInfo.toc.Identifier, "OnRoleChange"})

local function OnDescriptionComplete(unit, buffId)
   local info = {}
end
------------------------------
function KBB.setEnabled(flag)
	KBB.on = flag
	if flag then 
		for _, g in pairs(KBB.groups) do
			KBB.RescanGroup(g)
		end
	else
		for _, g in pairs(KBB.groups) do
			g:removeAllBars()
		end	
	end
end


------------------
-- refresh
------------------
local timeSinceLastUpdate = 0
local lastRefreshTime = Inspect.Time.Real()
local reaskCounter = 0
function KBB.Refresh()
	if not KBB.on then return end
	timeSinceLastUpdate = timeSinceLastUpdate + Inspect.Time.Real() - lastRefreshTime
	while (timeSinceLastUpdate > KBB.throttle) do
		for k, group in ipairs(KBB.groups) do
			reaskCounter = reaskCounter + KBB.throttle
			group:updateTimers(reaskCounter >= KBB.callsThrottle)
			if (reaskCounter >= KBB.callsThrottle) then reaskCounter = 0 end 
		end
		timeSinceLastUpdate = timeSinceLastUpdate - KBB.throttle
	end
	lastRefreshTime = Inspect.Time.Real()
end

table.insert(Event.System.Update.Begin, { KBB.Refresh, "KreiriBuffBars", "refresh" })

------------------
-- slash commands
------------------

local function slashcommand(argline)
	local args = Library.KLH.strsplit(argline, " ")
	if (args[1] == "addgroup") then
		local group = KBB.AddGroup()
		print(KBBLS("KBB_CREATED_GROUP", group.settings.name, group.settings.id))

	elseif (args[1] == "removegroup") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId (groupId)
		if group then
			if(KBB.RemoveGroupAtIndex(i)) then print(KBBLS("KBB_REMOVED_GROUP", group.settings.name, group.settings.id)) end
		end

	elseif (args[1] == "list") then
		local v = args[2]
		for _,group in pairs(KBB.groups) do
			if (v == "v") then
				print(KBBLS("KBB_GROUP_DESCRIPTION",
									group.settings.name,
									group.settings.id,
									group.settings.unit,
									group.settings.watchType,
									group.settings.filterType))
			else
				print(KBBLS("KBB_GROUP_DESCRIPTION_SHORT", 
									group.settings.name,
									group.settings.id))
			end
		end

	elseif (args[1] == "addfilter") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId (groupId)
		if (group) then
			local filter = table.concat(args, " ", 3)
			group:addFilter(filter)
			KBB.RescanGroup(group)
		end
	elseif (args[1] == "listfilters") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId (groupId)
		if (group) then
			print(KBBLS("KBB_FILTERS_IN_GROUP", group.settings.name, group.settings.id))
			for i, f in pairs(group.settings.filters) do
				print(i..": "..f.filterString)
			end
		end

	elseif (args[1] == "removefilter") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId (groupId)
		local index =  tonumber(args[3])
		if (group and index) then
			group:removeFilter(index)
-- 			KBB.RescanBuffs(group)
-- 			if(group.settings.sorts) then group:sortBars() end
-- 			group:alignBars()
			KBB.RescanGroup(group)
		end
		
    elseif (args[1] == "editfilter") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId (groupId)
		local index =  tonumber(args[3])
		if (group and index) then
			group:removeFilter(index)
			local filter = table.concat(args, " ", 4)
			group:addFilterAtIndex(filter, index)
-- 			KBB.RescanBuffs(group)
-- 			if(group.settings.sorts) then group:sortBars() end
-- 			group:alignBars()
			KBB.RescanGroup(group)
		end 
		
	elseif (args[1] == "removeallgroups") then
		KBB.RemoveAllGroups()
		print(KBBLS("KBB_REMOVED_ALL_GROUPS"))

	elseif (args[1] == "move") then
		local groupId = args[2]
	    local i, group = KBB.GroupWithId (groupId)
		if (group) then
			local x = tonumber(args[3])
			local y = tonumber(args[4])
			if (x and y) then
				group:move(x, y)
			end
		end

	elseif (args[1] == "anchors") then
		KBB.ToggleAnchors()

	elseif (args[1] == "setunit") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId (groupId)
		if (group) then
			local unit = args[3]
			if (KBB.units[unit]) then
				group:setUnit(unit)
--				KBB.RescanBuffs(group)
--				group:alignBars()
				KBB.RescanGroup(group)
			end
		end

	elseif (args[1] == "setwatchtype") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group and KBB.watchTypes[args[3]]) then
			group:setWatchType(args[3])
			KBB.RescanGroup(group)
		end

	elseif (args[1] == "setfiltertype") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if group and KBB.filterTypes[args[3]] then
			group:setFilterType(args[3])
			KBB.RescanGroup(group)
		end

	elseif (args[1] == "setbuffcolor") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local r, g, b = tonumber(args[3]), tonumber(args[4]), tonumber(args[5])
			if (r and g and b) then
				group:setBuffColor(r, g, b)
			end
		end

	elseif (args[1] == "setdebuffcolor") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local r, g, b = tonumber(args[3]), tonumber(args[4]), tonumber(args[5])
			if (r and g and b) then
				group:setDebuffColor(r, g, b)
			end
		end

	elseif (args[1] == "setbackalpha") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local alpha = tonumber(args[3])
			if (alpha) then
				group:setBackgroundAlpha(alpha)
			end
		end

	elseif (args[1] == "setbarwidth") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local w = tonumber(args[3])
			if (w) then
				group:setBarWidth(w)
			end
		end

	elseif (args[1] == "setbarheight") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local w = tonumber(args[3])
			if (w) then
				group:setBarHeight(w)
			end
		end

	elseif (args[1] == "setfontsize") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local w = tonumber(args[3])
			if (w) then
				group:setFontSize(w)
			end
		end
		
     elseif (args[1] == "where") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			print(groupId..": "..group.settings.position.x.." "..group.settings.position.y)
		end
		
	elseif (args[1] == "setcancel") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local flag 
			if (args[3] == "true")  then
				flag = true
			else
				flag = false
			end
			group.settings.cancelClick  = flag
		end
		
	elseif (args[1] == "setsort") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local flag
			if (args[3] == "asc" or args[3] == "desc")   then
				flag = true
				group.settings.sortOrder = args[3]
			end
			group.settings.sorts = flag or false
			if (flag) then 
				group:sortBars() 
				group:alignBars() 
			end
		end
	
	elseif (args[1] == "setgrowth") then
		local groupId = args[2]
		local i, group = KBB.GroupWithId(groupId)
		if (group) then
			local dir = args[3]
			if (KBB.growth[dir]) then
				group.settings.growth = dir
				if(group.settings.sorts) then group:sortBars() end
				group:alignBars()
			end
		end	
	
	elseif (args[1] == "on") then
		KBB.setEnabled(true)
		
	elseif (args[1] == "off") then
		KBB.setEnabled(false)
		
	elseif (args[1] == "r") then
		for _, g in pairs(KBB.groups) do
			g:removeAllBars()
			KBB.RescanGroup(g)
	 	end
	
	elseif (args[1] == "profiles") then
		for k, v in pairs(KBB.KBB_Settings_.settings) do
			print(k)
		end
		
	elseif (args[1] == "import")  then
		local source = args[2]
		local shard = Inspect.Shard().name 
		local character = Inspect.Unit.Detail("player").name
		local active =  character.."@"..shard
		if (KBB.KBB_Settings_.settings[source] == nil) then
			print(KBBLS("KBB_PROFILE_DN_EXIST"))
		elseif (source == active) then
			print(KBBLS("KBB_PROFILE_IS_ACTIVE", source))
		else
			print("importing groups from " .. source)
			KBB.importGroups(source, active, true)
		end 
	elseif (args[1] == "export") then
		local target = args[2]
		if string.len(target) > 0 then
			if KBB.KBB_Settings_.settings[target] then
				print(KBBLS("KBB_PROFILE_ALREADY_EXISTS", target))
			else  			
				local shard = Inspect.Shard().name 
				local character = Inspect.Unit.Detail("player").name
				local active =  character.."@"..shard
				local result = KBB.exportGroups(active, target)
				if (result)  then
					print(KBBLS("KBB_PROFILE_EXPORT_SUCCESS", active, target))
 				else
 					print(KBBLS("KBB_PROFILE_EXPORT_FAILED"))
				end
			end
		end
	elseif (args[1] == "ui") then
		if (not KBB.uiWindow) then KBB.MakeUI() end
--		local shard = Inspect.Shard().name
--		local character = Inspect.Unit.Detail("player").name
--		local thisChar = character .. "@" .. shard
--		Library.KLH.tdump(KBB.KBB_Settings_)
		KBB.ToggleUI()
	end	
end

table.insert(Command.Slash.Register("kbb"), {slashcommand, addonInfo.toc.Identifier, "slashcommand"})
