local addonInfo, addonTable = ...
if not addonTable.KBB then addonTable.KBB = {} end
local KBB = addonTable.KBB

local KBBDebug = KBB.debug

function KBB.AnalyzeFilterString(filterstring)
	local left, right, op
	local ob, oe
	for _, v in ipairs(KBB.filterOps) do
		ob, oe = string.find(filterstring, v)
		if ob and oe then break end
	end

	if ob and oe then
		left = string.sub(filterstring, 1, ob - 1)
		op = string.sub(filterstring, ob, oe)
		right = string.sub(filterstring, oe + 1)
	else
		left = filterstring
	end
	return left, op, right
end

function KBB.MakeFilter(filterString)
	if not filterString then return nil end
	local filter  = {}
	filter.filterString = filterString
	local level1 = Library.KLH.strsplit(filterString, "&")
	for i, chunk  in pairs (level1) do
		filter[i] = {}
		if (chunk == "my" or chunk == "!my")  then
			filter[i].ftype = "my"
			filter[i].value =  (chunk == "my")
		elseif (chunk == "dis" or chunk == "!dis")  then
			filter[i].ftype = "dis"
			filter[i].value = (chunk == "dis")
		 elseif (chunk == "t==nil") then
			filter[i].ftype = "t"
			filter[i].op = "==nil"
		elseif (chunk == "t!=nil") then
			filter[i].ftype = "t"
			filter[i].op = "!=nil"
		else
			local left, op, right = KBB.AnalyzeFilterString(chunk)
			if (not KBB.filters[left]) then
				print(KBB.localString("KBB_ERROR_IN_FILTER", chunk))
				print(KBB.localString("KBB_UNKNOWN_FILTER", left or ""))
				return nil
			end
			if not (Library.KLH.tableContains(KBB.filters[left], op)) then
				print(KBB.localString("KBB_ERROR_IN_FILTER", chunk))
				print(KBB.localString("KBB_UNKNOWN_OPERATOR", op or ""))
				return nil
			end
			if not (right) or right == "" then
				print(KBB.localString("KBB_ERROR_IN_FILTER", chunk))
				print(KBB.localString("KBB_FILTER_VALUE_NIL"))
				return nil
			end
			filter[i].ftype = left
			filter[i].op = op
			if left == "n" then right = string.lower(right) end
			filter[i].value = right
		end
	end
	return filter
end

function KBB.MatchFilterChunk(buffinfo, chunk)
	local result = false
	if chunk.ftype == "my" then
		if ((buffinfo.caster == KBB.playerID) == chunk.value) then
			result =  true
		else
			result = false
		end
	elseif chunk.ftype == "n" then
		if chunk.op == "=" then
-- 			result = (string.lower(buffinfo.name) == string.lower(chunk.value))
			result = (string.lower(buffinfo.name) == chunk.value)
-- 			KBBDebug(chunk.value .. " = " .. buffinfo.name .. " ? " .. tostring(result))
		elseif chunk.op == "~=" then
-- 			result = not not string.find(string.lower(buffinfo.name), string.lower(chunk.value))
			result = not not string.find(string.lower(buffinfo.name), chunk.value)
-- 			KBBDebug(chunk.value .. " ~= " .. buffinfo.name .. " ? " .. tostring(result))
		end
	elseif chunk.ftype == "t" then
-- 		KBBDebug("duration: "..tostring(buffinfo.duration))
		if not buffinfo.duration then result = (chunk.op == "==nil")
		elseif (chunk.op == "!=nil") then
			result = true
		else
			local d = math.ceil(buffinfo.duration)
			if (chunk.op == "<") then
				result = d < tonumber(chunk.value)
			elseif   (chunk.op == ">") then
				result = d > tonumber(chunk.value)
			elseif (chunk.op == "<=") then
				result = d <= tonumber(chunk.value)
			elseif (chunk.op == ">=") then
				result = d >= tonumber(chunk.value)
			end
		end
	end
	return result
end

function KBB.MatchFilter(buffinfo, filter)
	local result = true
-- 	KBBDebug("matching ["..buffinfo.name.."] against ["..filter.filterString.."]")
	for _, chunk in ipairs(filter) do   -- skips ["filterString"] because it's not an integer
-- 		KBBDebug("chunk: "..Utility.Serialize.Inline(chunk))
		result  = result and  KBB.MatchFilterChunk(buffinfo, chunk)
		if not result then break end
	end
-- 	KBBDebug("filter result: "..tostring(result))
	return result
end

function KBB.MatchBuff(buffinfo, filters, filterType)
-- 	KBBDebug("began matching for "..buffinfo.name)
-- 	KBBDebug("filter type: "..filterType)
	local result = (filterType == "blacklist")
	for i, f in pairs(filters) do
		if KBB.MatchFilter(buffinfo, f) then
			result = not result
			break
		end
	end
-- 	KBBDebug("buff result: "..tostring(result))
	return result
end
