local addonInfo, addonTable = ...
if not addonTable.KBB then addonTable.KBB = {} end
local KBB = addonTable.KBB
local KBB_Defaults = KBB.defaults

KBB.KBB_Group = {}
local KBB_Group = KBB.KBB_Group
KBB_Group.__index = KBB_Group

local KBBDebug = KBB.debug

if not KBB.context then
	KBB.context = UI.CreateContext("KBB_Context")
	KBB.context:SetAllPoints(UIParent)
	KBB.context:SetLayer(1)
end

function KBB_Group:loadDefaults()
	local name = self.settings.name
	local id = self.settings.id
	self.settings = Library.KLH.deepcopy(KBB_Defaults.group)
	for key, value in pairs(self.settings["fontColor"]) do
		self.settings["debuffFontColor"][key] = value
	end
	self.unitID = Inspect.Unit.Lookup("player")
	self.settings.id = id
--	self.settings.name = name
	self.settings.name = id
	self.settings.version = addonInfo.toc.Version
	self.frame:SetPoint("CENTER", UIParent, "CENTER", self.settings.position.x, self.settings.position.y)
	self.frame:SetWidth(self.settings.barWidth)
	self.frame:SetHeight(self.settings.barHeight)
end

function KBB_Group:updateFrame()
	self.frame:SetWidth(self.settings.barWidth)
	self.frame:SetHeight(self.settings.barHeight)
end
function KBB_Group.new()
--	local group = UI.CreateFrame("Frame", "KBB_Group", KBB.context)
	local group = {}
	setmetatable(group, KBB_Group)
	group.settings = {}

	group.bars = {}

--	group.settings.filters = {}

	group.frame = UI.CreateFrame("Frame", "KBB_GroupFrame", KBB.context)
	group:loadDefaults()
	group.frame:SetWidth(group.settings.barWidth)
	group.frame:SetHeight(group.settings.barHeight)
	group.frame:SetVisible(true)
--	group.frame:SetBackgroundColor(unpack(KBB.anchorColor))

	group.anchor = UI.CreateFrame("Frame", "Anchor", group.frame)
	group.anchor:SetPoint("TOPLEFT", group.frame, "TOPLEFT", 0, 0)
	group.anchor:SetPoint("BOTTOMRIGHT", group.frame, "BOTTOMRIGHT", 0, 0)
	group.anchor:SetBackgroundColor(unpack(KBB.anchorColor))
	group.anchor:SetVisible(KBB.showAnchors)
	group.anchor:SetLayer(-1)
	group.anchor.text = UI.CreateFrame("Text", "Text", group.anchor)
	group.anchor.text:SetPoint("TOPCENTER", group.anchor, "TOPCENTER", 0, 0)
	group.anchor.text:SetFontSize(group.settings.textFontSize)
--	group.anchor.text:SetPoint("TOPLEFT", group.anchor, "TOPLEFT", 0, 0)
--	group.anchor.text:SetPoint("BOTTOMLEFT", group.anchor, "BOTTOMLEFT", 0, 0)
	group.anchor.text:SetWordwrap(false)
	
	group.anchor.text:SetFontColor(1, 1, 1, 1)

	group.barsContainer = UI.CreateFrame("Frame", "BarsContainer", group.frame)
	group.barsContainer:SetVisible(true)
	return group
end

function KBB_Group:setId(n)
	self.settings.id = KBB_Defaults.group.name..tostring(n)
	if not self.settings.name or self.settings.name == KBB_Defaults.group.name then
		self.settings.name = self.settings.id
--		self:updateAnchorText()
	end
end

function KBB_Group:setUnit(unit)
	self.settings.unit = unit
--	self:updateAnchorText()
	local unitID = Inspect.Unit.Lookup(unit)
-- 	print(unitID)
	self.unitID = unitID
end

function KBB_Group:setFilterType(ft)
	self.settings.filterType = ft
--	self:updateAnchorText()
end

function KBB_Group:setWatchType(ft)
	self.settings.watchType = ft
--	self:updateAnchorText()
end

function KBB_Group:setName(name)
	self.settings.name = name
	self:updateAnchorText()
end

function KBB_Group:setBuffColor(r, g, b)

	self.settings.buffColor.r = r
	self.settings.buffColor.g = g
	self.settings.buffColor.b = b

	self:updateColors()
end

function KBB_Group:getBuffColor()
	return self.settings.buffColor.r, self.settings.buffColor.g, self.settings.buffColor.b
end

function KBB_Group:setDebuffColor(r, g, b)
	self.settings.debuffColor.r = r
	self.settings.debuffColor.g = g
	self.settings.debuffColor.b = b

	self:updateColors()
end

function KBB_Group:getDebuffColor()
	return self.settings.debuffColor.r, self.settings.debuffColor.g, self.settings.debuffColor.b
end

function KBB_Group:setBackgroundColor(r, g, b)
	self.settings.backgroundColor.r = r
	self.settings.backgroundColor.g = g
	self.settings.backgroundColor.b = b

	self:updateColors()
end

function KBB_Group:getBackgroundColor()
	return self.settings.backgroundColor.r, self.settings.backgroundColor.g, self.settings.backgroundColor.b
end

function KBB_Group:setBackgroundAlpha(a)
	self.settings.backgroundAlpha = a
	self:updateBackgroundAlpha()
end

function KBB_Group:getBackgroundAlpha()
	return self.settings.backgroundAlpha
end

function KBB_Group:setBarWidth(w)
	if w then
		self.settings.barWidth = w
		self.frame:SetWidth(w)
		for _, bar in pairs(self.bars) do
			bar:setWidth(w)
			bar:fitText(self.settings.stackMargin)
		end
	end
end

function KBB_Group:setBarHeight(h)
	if h then
		self.settings.barHeight = h
		self.frame:SetHeight(h)
		for _, bar in pairs(self.bars) do
			bar:setHeight(h)
		end
		self:alignBars()
	end
end

function KBB_Group:setFontSize(h)
	if h then
		self.settings.textFontSize = h
		self.settings.stackFontSize = h
		self.settings.timerFontSize = h
		for _, bar in pairs(self.bars) do
			bar:setFontSize(h)
			bar:updateLayout(self.settings.barLayout)
			bar:fitText(self.settings.stackMargin)
		end
	end
end

function KBB_Group:setTextFontSize(h)
	if h then
		self.settings.textFontSize = h
		for _, bar in pairs(self.bars) do
			bar:setTextFontSize(h)
		end
	end
end

function KBB_Group:setStackFontSize(h)
	if h then
		self.settings.stackFontSize = h
		for _, bar in pairs(self.bars) do
			bar:setStackFontSize(h)
		end
	end
end

function KBB_Group:setTimerFontSize(h)
	if h then
		self.settings.timerFontSize = h
		for _, bar in pairs(self.bars) do
			bar:setTimerFontSize(h)
			bar:updateLayout(self.settings.barLayout)
		end
	end
end

function KBB_Group:setFontColor(r, g, b)
	self.settings.fontColor.r = r
	self.settings.fontColor.g = g
	self.settings.fontColor.b = b
	for _, bar in pairs(self.bars) do
--		bar:setFontColor(r, g, b)
		bar:updateFontColor()
	end
end

function KBB_Group:getFontColor()
	return self.settings.fontColor.r, self.settings.fontColor.g, self.settings.fontColor.b
end

function KBB_Group:setDebuffFontColor(r, g, b)
	self.settings.debuffFontColor.r = r
	self.settings.debuffFontColor.g = g
	self.settings.debuffFontColor.b = b
	for _, bar in pairs(self.bars) do
		--		bar:setFontColor(r, g, b)
		bar:updateFontColor()
	end
end

function KBB_Group:getDebuffFontColor()
	return self.settings.debuffFontColor.r, self.settings.debuffFontColor.g, self.settings.debuffFontColor.b
end

function KBB_Group:addBar(bar)
	if not bar then return end
	if not self.bars then self.bars = {} end
	KBBDebug("adding bar")
	table.insert(self.bars, bar)
	KBBDebug("added bar, ".. #(self.bars).. " bars in group now")
	bar.parentGroup = self
	bar.container:SetParent(self.barsContainer)
	bar:setWidth(self.settings.barWidth)
	bar:setHeight(self.settings.barHeight)

	bar:setTextFontSize(self.settings.textFontSize)
	bar:setStackFontSize(self.settings.stackFontSize)
	bar:setTimerFontSize(self.settings.timerFontSize)
	bar:updateLayout(self.settings.barLayout)

	local yoffset = #(self.bars)*(self.settings.barHeight + self.settings.spacing)
	if self.settings.growth == "up" then yoffset = -yoffset end
	bar.frame:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 0, yoffset)

	bar:updateColor()
	bar:updateFontColor()
	bar.container:SetVisible(true)
end

function KBB_Group:removeBarWithBuffId(buffId)
	local aBar
	local left, right, top, bottom
	local mouse = Inspect.Mouse()
	for k, bar in ipairs(self.bars) do
		if (bar.buffInfo.buffId == buffId) then
-- 			left =  bar.frame:GetLeft()
-- 			right =  bar.frame:GetRight()
-- 			top =  bar.frame:GetTop()
-- 			bottom =   bar.frame:GetBottom()
-- 			if (mouse.x <= right and mouse.x >= left and
-- 				mouse.y <= bottom and mouse.y >= top) then  					 
-- 					KBB.hideBuffTooltip()
-- 			end
			bar:reset()
-- 			bar.parentGroup = nil
			table.insert(KBB.barcache, bar)
			KBBDebug("put bar to cache, "..#(KBB.barcache).." bars in cache now")
			table.remove(self.bars, k)
			KBBDebug("removed bar from group, "..#(self.bars).." bars in group remaining")
			aBar = bar
		end
	end
	return aBar
end

function KBB_Group:removeAllBars()
	for _, bar in ipairs(self.bars) do
		bar:reset()
--		bar.frame:SetParent(KBB.context)
		table.insert(KBB.barcache, bar)
	end
	self.bars = {}
end

-- function KBB_Group:updateTimers(reask)
-- 	for _, bar in pairs(self.bars) do
-- 		bar:updateTimer(reask)
-- 	end
-- end

function KBB_Group:updateTimers(reask)
	if not self.unitID then return end
	if #(self.bars) == 0 then return end
	if (not reask) then -- update timers by subtracting update interval from remaining
		for _, bar in pairs(self.bars) do
			bar:update()
		end
	else -- update bar timers with current buff info
		local buffIds = {}		
		local empty = true
		for _, bar in pairs(self.bars) do -- make buff list
			buffIds[bar.buffInfo.buffId] = true
			if empty then empty = false end
		end
	 	if empty then return end
	 	
-- 	 	local unitId = Inspect.Unit.Lookup(self.settings.unit)
	 	
		local buffInfo = Inspect.Buff.Detail(self.unitID, buffIds) -- ask server for current buff details
		if  buffInfo then 		
			for _, bar in pairs(self.bars) do
				bar:update(buffInfo[bar.buffInfo.buffId]) -- update timers using current info
			end
		end
-- 		local unitId = Inspect.Unit.Lookup(self.settings.unit)
-- 		local buffInfo = Inspect.Buff.Detail(unitId, self.buffIds) 
-- 		for _, bar in pairs(self.bars) do
-- 			bar:update(buffInfo[bar.buffInfo.buffId])
-- 		end
	end
end

function KBB_Group:barWithBuffId(buffId)
	for _, bar in ipairs(self.bars) do
		if (bar.buffInfo.buffId == buffId) then
			return bar
		end
	end
	return nil
end

function KBB_Group:alignBars()
	local yOffset = 0
	local i =0
	for _, bar in ipairs(self.bars) do
		yOffset = i*(self.settings.barHeight + self.settings.spacing)
		if self.settings.growth == "up" then yOffset = 0 - yOffset end
		bar.frame:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 0, yOffset)
		
		i = i + 1
	end

-- 	KBB.hideBuffTooltip()
	if KBB.characterDB and KBB.characterDB.settings.useNativeTooltip then
			Command.Tooltip(nil)
		else
			KBB.hideBuffTooltip()
		end
	for _, bar in ipairs(self.bars) do
		bar:showTooltip()
	end
end

function KBB_Group:updateAnchorText()
	local settings = self.settings
	
--	local text = settings.id.." : "..settings.unit.." : "..settings.watchType .. " : " .. settings.filterType
	
	self.anchor.text:SetText(settings.name)
end

function KBB_Group:updateAnchorColor()
	self.anchor:SetBackgroundColor(unpack(KBB.anchorColor))
end


function KBB_Group:move(x, y)
-- print("move called for "..self.settings.name, x, y)
	
	if ( x ) then self.settings.position.x = x end
	if ( y ) then self.settings.position.y = y end
	self.frame:SetPoint("CENTER", UIParent, "CENTER", self.settings.position.x, self.settings.position.y)
end


function KBB_Group:updateColors()
	 for _, bar in pairs(self.bars) do
		bar:updateColor()
	end
end

function KBB_Group:updateBackgroundAlpha()
	for _, bar in pairs(self.bars) do
		bar:updateBackgroundAlpha()
	end
end

function KBB_Group:addFilter(filterstring)
	local filter = KBB.MakeFilter(filterstring)
	if (filter) then
		table.insert(self.settings.filters, filter)
	end
end

function KBB_Group:addFilterAtIndex(filterstring, index)
	local filter = KBB.MakeFilter(filterstring)
	if (filter) then
		table.insert(self.settings.filters, index, filter)
	end
end

function KBB_Group:removeFilter(index)
	if (index <= #(self.settings.filters)) then table.remove(self.settings.filters, index) end
end

--local function barCompare (barA, barB, selector, mode)
--	if (barA.buffInfo[selector] and barB.buffInfo[selector]) then
--		if (selector == "name") then
--			return (barA.buffInfo[selector]:upper() < barA.buffInfo[selector]:upper()) and mode
--		else
--			return (barA.buffInfo[selector] < barA.buffInfo[selector]) and mode
--		end
--	else
--		return barA.buffInfo[selector] ~= nil
--	end
--end

function KBB_Group:sortBars()
	if (self.settings.sortOrder == "asc") then  
		table.sort(self.bars, function (a, b) return (a > b) end)
	elseif (self.settings.sortOrder == "desc") then
		table.sort(self.bars, function (a, b) return (a < b) end)
	end
end
