-- here will be GUI

local addonInfo, addonTable = ...
local addonId = addonInfo.toc.Identifier
if not addonTable.KBB then addonTable.KBB = {} end
local KBB = addonTable.KBB

local KBBUI = {}
local KBBLS = KBB.localString
--------------------------------------------------
-----------------------------------

local ACTIVE_FONT_R = 1--0.86
local ACTIVE_FONT_G = 1--0.86
local ACTIVE_FONT_B = 1--0.63
local BORDER_R = .48 -- 0.3
local BORDER_G = .48 -- 0.5
local BORDER_B = .4 -- 0.5
local HIGHLIGHT_TAB_FONT_R = 1
local HIGHLIGHT_TAB_FONT_G = 1
local HIGHLIGHT_TAB_FONT_B = 1

local WINDOW_WIDTH = 900
local WINDOW_HEIGHT = 580

--local FILTERS_HEIGHT = 100

local RENAME_WINDOW_HEIGHT = 150
local RENAME_WINDOW_WIDTH = 300

local RGB_RANGE = 255

local EDITFIELD_ALPHA  = 0.5

local MAX_BAR_WIDTH = 500
local MAX_BAR_HEIGHT = 100

local MIN_FONT_SIZE = 10
local MAX_FONT_SIZE = 40
local UI_FONT_SIZE = 15

local FILTER_MODE_NONE = KBBLS("UI_FILTER_MODE_NONE")
local FILTER_MODE_EDIT = KBBLS("UI_FILTER_MODE_EDIT")
local FILTER_MODE_ADD = KBBLS("UI_FILTER_MODE_ADD")

local MINIMAP_BUTTON_HEIGHT = 30    -- just a number to use while saved variables are loading
local MINIMAP_BUTTON_WIDTH = 30

if not KBB.context then
	KBB.context = UI.CreateContext("KBB_Context")
	KBB.context:SetAllPoints(UIParent)
	KBB.context:SetLayer(1)
end
KBB.uiContext = UI.CreateContext("KBB_UIContext")
KBB.uiContext:SetAllPoints(UIParent)
KBB.uiContext:SetLayer(2)

KBB.mmbContext = UI.CreateContext("KBB_mmbContext") 
-- local stratas = KBB.mmbContext:GetStrataList()
-- for k, v in ipairs(stratas) do print (k, v) end
-- KBB.mmbContext:SetStrata("hud")

--------------------------------------------------
----------- tooltip ---------------------
----------------------------------------------------------
KBB.screenSize = {}
KBBUI.MakeTooltip = function()
	Command.System.Watchdog.Quiet()
	KBB.tooltipFrame = UI.CreateFrame("Frame", "KBB_Tooltip", KBB.context)
	KBB.tooltipFrame:SetLayer(100)
	KBB.tooltipFrame:SetVisible(false)
	local tooltipWidth = 194
	KBB.tooltipFrame:SetWidth(tooltipWidth)
	KBB.tooltipFrame:SetBackgroundColor(0, 0, 0, .8)
	
	KBB.tooltipFrame.title = UI.CreateFrame("Text", "Title", KBB.tooltipFrame)
	KBB.tooltipFrame.description = UI.CreateFrame("Text", "Description", KBB.tooltipFrame)
	local title = KBB.tooltipFrame.title
	local description = KBB.tooltipFrame.description
	
	KBB.tooltipFrame.xpadding = 4.0
	KBB.tooltipFrame.ypadding = 4.0
	
	title:SetWidth(tooltipWidth - 2 * KBB.tooltipFrame.xpadding)
	description:SetWidth(tooltipWidth - 2 * KBB.tooltipFrame.xpadding)
	
	title:SetPoint("TOPLEFT", KBB.tooltipFrame, "TOPLEFT", KBB.tooltipFrame.xpadding, KBB.tooltipFrame.ypadding)
	title:SetFontSize(KBB.tooltipFontSize)
	title:SetWordwrap(true)
	description:SetPoint("BOTTOMLEFT", KBB.tooltipFrame, "BOTTOMLEFT", KBB.tooltipFrame.xpadding, -KBB.tooltipFrame.ypadding)
	description:SetFontSize(KBB.tooltipFontSize)
	description:SetWordwrap(true)
	
	title:SetFontColor(223/RGB_RANGE, 206/RGB_RANGE, 114/RGB_RANGE)
end

KBBUI.MakeTooltip()

function KBB.showBuffTooltip(info)
	if not info then return end

	KBB.tooltipFrame.title:SetText(info.name or "")
	KBB.tooltipFrame.description:SetText(info.description or "")



	KBB.tooltipFrame:SetHeight(KBB.tooltipFrame.description:GetHeight()
			+ KBB.tooltipFrame.title:GetHeight() +
			3 * KBB.tooltipFrame.ypadding)

	local loc = Inspect.Mouse()

	local ttRect = {}
	ttRect.width =  KBB.tooltipFrame:GetWidth()
	ttRect.height = KBB.tooltipFrame:GetHeight()
	ttRect.x = loc.x - ttRect.width
	ttRect.y = loc.y - ttRect.height

	if ttRect.x < 0 then
		ttRect.x = 0
	elseif loc.x > KBB.screenSize.w then
		ttRect.x = KBB.screenSize.w - ttRect.width
	end

	if ttRect.y < 0 then
		ttRect.y = 0
	elseif loc.y > KBB.screenSize.h then
		ttRect.y = KBB.screenSize.h - ttRect.height
	end


	KBB.tooltipFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ttRect.x, ttRect.y)
	KBB.tooltipFrame:SetVisible(true)
end

function KBB.hideBuffTooltip()
	KBB.tooltipFrame:SetVisible(false)
end

KBB.tooltipFrame.SetFontSize = function(self, size)
	self.title:SetFontSize(size)
	self.description:SetFontSize(size)
end

----------------------------------------------------------
-- minimap button
-----------------------------------------------------------------------------------
KBBUI.MakeMMB = function()
	KBB.minimapButton = UI.CreateFrame("Frame", "KBB_mmb", KBB.mmbContext)
	local mmb = KBB.minimapButton
	
	mmb:SetLayer(UI.Native.MapMini:GetLayer())
	mmb:SetWidth(MINIMAP_BUTTON_WIDTH)
	mmb:SetHeight(MINIMAP_BUTTON_HEIGHT)
	
	mmb.borderNormal = UI.CreateFrame("Texture", "KBB_mmbBorderN", mmb)
	mmb.borderNormal:SetTexture(addonId, "Media/ring_orange.png")
	mmb.borderNormal:SetAllPoints(mmb)
	mmb.borderNormal:SetLayer(1)
	
	mmb.borderBright = UI.CreateFrame("Texture", "KBB_mmbBorderB", mmb)
	mmb.borderBright:SetTexture(addonId, "Media/ring_orange_flared.png")
	mmb.borderBright:SetAllPoints(mmb)
	mmb.borderBright:SetLayer(1)
	mmb.borderBright:SetVisible(false)
	
	mmb.buttonImage = UI.CreateFrame("Texture", "KBB_mmbImage", mmb)
	mmb.buttonImage:SetTexture(addonId, "Media/kbb_mm_icon.png")
	mmb.buttonImage:SetAllPoints(mmb)
	mmb.buttonImage:SetLayer(0)
	
	mmb.Event.MouseIn = function(button)
			mmb.borderNormal:SetVisible(false)
			mmb.borderBright:SetVisible(true)
	end
	mmb.Event.MouseOut = function(button)
		mmb.borderNormal:SetVisible(true)
		mmb.borderBright:SetVisible(false)
	end
	mmb.Event.LeftClick = function(button)
	-- 	print("mmb.Event.LeftClick")
		KBB.ToggleUI()
	end

end
KBBUI.MakeMMB()
--Library.KLH.attachButtonToMinimap(mmb, 225, 5)

-----------------------------------
-------- helper methods ----------
----------------------------------

KBBUI.MakeRadioButton =  function (name, parent, text)
	local button = Library.LibSimpleWidgets.RadioButton(name, parent)
	button:SetText(text)
	button:SetFontSize(UI_FONT_SIZE)
	button:SetFontColor(1, 1, 1, 1)
	button.KLHshadow = Library.KLH.makeShadow(button.label, 0, 0, 0, 1, -2, 2)
	return button
end

KBBUI.MakeLabel =  function (name, parent, text)
	local label = UI.CreateFrame("Text", name, parent)
	label:SetText(text)
	label:SetFontSize(UI_FONT_SIZE)
	label:SetFontColor(ACTIVE_FONT_R, ACTIVE_FONT_G, ACTIVE_FONT_B)
	label.KLHshadow = Library.KLH.makeShadow(label, 0, 0, 0, 1, -2, 2)
	return label
end

--SetSliderEnabled = function(self, enabled)
--	self.enabled = enabled
--	if enabled then
--		self.current:SetFontColor(1, 1, 1, 1)
--	else
--		self.current:SetFontColor(0.5, 0.5, 0.5, 1)
--	end
--	if self.editor then
--		self.editor:SetVisible(enabled)
--	end
--	self.slider:SetEnabled(enabled)
--end


--SetRadiogroupEnabled =  function (radiogroup, value)
--	for _, r in pairs(radiogroup.radioButtons) do
--		r:SetEnabled(value)
--	end
--end

---- this is what I get for trusting third-party libraries. Seriously,
---- newly-added to LibSimpleWidgets (1.9.0) SetSelectedIndex for radio group
---- does not deselect other radiobuttons!
--SetRadiogroupSelectedIndex = function(radiogroup, index, silent)
--	local radioButton = radiogroup.radioButtons[index]
--	radioButton:SetSelected(true, silent)
--	for _, r in pairs(radiogroup.radioButtons) do
--		if r ~= radioButton then r:SetSelected(false, true) end
--	end
--end

KBBUI.SetSliderValue = function(slider, valueA)
	local minA, maxA = slider:GetRange()
	local value = valueA
	if (minA > value) then
		value = minA
	elseif (value > maxA) then
		value = maxA
	end
	slider:SetPosition(value)
end


KBBUI.OnSliderWheelBack = function(slider)
	if not slider:GetEnabled() then return end
	local value = slider:GetPosition()
	KBBUI.SetSliderValue(slider, value - 1)
end

KBBUI.OnSliderWheelForward = function(slider)
	if not slider:GetEnabled() then return end
	local value = slider:GetPosition()
	KBBUI.SetSliderValue(slider, value + 1)
end

local function resizeCurrentForRange(current, max)
  local oldCurrent = current:GetText()
  current:SetText(tostring(max))
  current:ClearWidth()
  current:SetWidth(current:GetWidth())
  current:SetText(oldCurrent)
end

KBBUI.MakeSlider =  function (name, parent, minA, maxA, width, pos)
	local slider = Library.LibSimpleWidgets.Slider(name, parent)
	slider:SetEditable(true)
	slider:SetRange(minA, maxA)
	local position = pos
	if position == nil then position = minA end
	slider:SetPosition(position)
	slider:SetWidth(width)
	slider.Event.WheelForward = KBBUI.OnSliderWheelForward
	slider.Event.WheelBack = KBBUI.OnSliderWheelBack
--	slider.SetEnabled = SetSliderEnabled
	slider.editor:SetBackgroundColor(0, 0, 0, 1)
	slider.current:SetBackgroundColor(0, 0, 0, 1)
--	slider.current:SetFontColor(0, 0, 0, 1)
	Library.LibSimpleWidgets.SetBorder(slider.editor, 1, 0.5, 0.5, 0.5, false)
--	slider.editor:SetLayer(-1)
	if KBB.debugMode then
		slider:SetBackgroundColor(0.7, 0.8, 0, 0.5)
	end
	return slider
end

KBBUI.MakeColorSwatch = function(name, parent, width, height)
	local swatch = UI.CreateFrame("Frame", parent:GetName() .. "_" .. name, parent)
	swatch:SetWidth(width)
	swatch:SetHeight(height)
	return swatch
end

KBBUI.MakeColorsPreviewWidget = function(name, parent, title, barwidth, barheight, bartext, fontsize)
	local container = UI.CreateFrame("Frame", parent:GetName() .. "_" .. name .. ".container", parent)

	container.titleLabel = KBBUI.MakeLabel(parent:GetName() .. "_" .. name .. ".titleLabel", container, title)
	container.titleLabel:SetPoint("TOPLEFT", container, "TOPLEFT")

	container.buffBarBg = UI.CreateFrame("Frame", parent:GetName() .. "_" .. name ..".buffBarBg", container)
	container.buffBarBg:SetPoint("TOPLEFT", container.titleLabel, "BOTTOMLEFT", 0, 3)
	container.buffBarBg:SetLayer(-1)
	container.buffBarBg:SetWidth(barwidth)
	container.buffBarBg:SetHeight(barheight)

	container.buffBarFg = UI.CreateFrame("Frame", parent:GetName() .. "_" .. name .. ".buffBarFg", container.buffBarBg)
	container.buffBarFg:SetPoint("TOPLEFT", container.buffBarBg, "TOPLEFT")
	container.buffBarFg:SetPoint("BOTTOMLEFT", container.buffBarBg, "BOTTOMLEFT")
	container.buffBarFg:SetPoint("TOPRIGHT", container.buffBarBg, "TOPCENTER")
	container.buffBarFg:SetLayer(1)

	container.buffText = UI.CreateFrame("Text", parent:GetName() .. "_" .. name .. ".buffText", container.buffBarBg)
	container.buffText:SetText(bartext)
	container.buffText:SetFontSize(fontsize)
	container.buffText:SetPoint("CENTER", container.buffBarBg, "CENTER")
	container.buffText:SetLayer(2)

	container.debuffBarBg = UI.CreateFrame("Frame", parent:GetName() .. "_" .. name .. ".debuffBarBg", container)
	container.debuffBarBg:SetPoint("TOPLEFT", container.buffBarBg, "BOTTOMLEFT", 0, 5)
	container.debuffBarBg:SetLayer(-1)
	container.debuffBarBg:SetWidth(barwidth)
	container.debuffBarBg:SetHeight(barheight)

	container.debuffBarFg = UI.CreateFrame("Frame", parent:GetName() .. "_" .. name .. ".debuffBarFg", container.debuffBarBg)
	container.debuffBarFg:SetPoint("TOPLEFT", container.debuffBarBg, "TOPLEFT")
	container.debuffBarFg:SetPoint("BOTTOMLEFT", container.debuffBarBg, "BOTTOMLEFT")
	container.debuffBarFg:SetPoint("TOPRIGHT", container.debuffBarBg, "TOPCENTER")
	container.debuffBarFg:SetLayer(1)

	container.debuffText = UI.CreateFrame("Text", parent:GetName() .. "_" .. name .. ".debuffText", container.debuffBarBg)
	container.debuffText:SetText(bartext)
	container.debuffText:SetFontSize(fontsize)
	container.debuffText:SetPoint("CENTER", container.debuffBarBg, "CENTER")
	container.debuffText:SetLayer(2)

	local fh = container.debuffBarBg:GetBottom() - container.titleLabel:GetTop()
	local fw = math.max(container.debuffBarBg:GetRight(), container.titleLabel:GetRight())
			- math.min(container.titleLabel:GetLeft(), container.debuffBarBg:GetLeft())

	container:SetHeight(fh)
	container:SetWidth(fw)

	return container
end


local function SetCSGEnabled(csg, state)
	csg.rSlider:SetEnabled(state)
	csg.gSlider:SetEnabled(state)
	csg.bSlider:SetEnabled(state)
end

KBBUI.MakeColorSliderGroup = function(name, parent, sliderWidth, title)
	local container = UI.CreateFrame("Frame", parent:GetName().."_"..name..".container", parent)

	container.titleLabel = KBBUI.MakeLabel(parent:GetName() .. "_" .. name .. ".titleLabel", container, title)
	container.titleLabel:SetPoint("TOPLEFT", container, "TOPLEFT")

	container.rLabel = KBBUI.MakeLabel(parent:GetName() .. "_" .. name .. ".rLabel", container, KBBLS("UI_COLOR_RED_LABEL_TEXT"))
	container.rLabel:SetPoint("TOPLEFT", container.titleLabel, "BOTTOMLEFT", 0, 3)

	container.gLabel = KBBUI.MakeLabel(parent:GetName() .. "_" .. name .. ".gLabel", container, KBBLS("UI_COLOR_GREEN_LABEL_TEXT"))
	container.gLabel:SetPoint("TOPLEFT", container.rLabel, "BOTTOMLEFT", 0, 3)

	container.bLabel = KBBUI.MakeLabel(parent:GetName() .. "_" .. name .. ".bLabel", container, KBBLS("UI_COLOR_BLUE_LABEL_TEXT"))
	container.bLabel:SetPoint("TOPLEFT", container.gLabel, "BOTTOMLEFT", 0, 3)

	local lw = math.max(math.max(container.rLabel:GetWidth(), container.gLabel:GetWidth()), container.bLabel:GetWidth())
	local dx = 5
	container.rSlider =  KBBUI.MakeSlider(parent:GetName() .. "_" .. name .. ".rSlider", container, 0, RGB_RANGE, sliderWidth, math.floor(0.5))
	container.rSlider:SetPoint("BOTTOMLEFT", container.rLabel, "BOTTOMLEFT", lw + dx, 0)

	container.gSlider = KBBUI.MakeSlider(parent:GetName() .. "_" .. name .. "gSlider", container, 0, RGB_RANGE, sliderWidth, math.floor(0.5))
	container.gSlider:SetPoint("BOTTOMLEFT", container.gLabel, "BOTTOMLEFT", lw + dx, 0)

	container.bSlider = KBBUI.MakeSlider(parent:GetName() .. "_" .. name .. ".bSlider", container, 0, RGB_RANGE, sliderWidth, math.floor(0.5))
	container.bSlider:SetPoint("BOTTOMLEFT", container.bLabel, "BOTTOMLEFT", lw + dx, 0)

	local fh = container.bLabel:GetBottom() - container.titleLabel:GetTop()
	local fw = container.rSlider:GetRight() - container.titleLabel:GetLeft()

	container:SetHeight(fh)
	container:SetWidth(fw)

	container.SetEnabled = SetCSGEnabled

	return container
end

KBBUI.MakeCheckbox = function(name, parent, text)
	local checkbox = Library.LibSimpleWidgets.Checkbox(name, parent)
	checkbox:SetText(text)
	checkbox:SetFontSize(UI_FONT_SIZE)
	--   checkbox:SetFontColor(ACTIVE_FONT_R, ACTIVE_FONT_G, ACTIVE_FONT_B, 1)
	checkbox:SetFontColor(1, 1, 1, 1)
	checkbox.KLHshadow = Library.KLH.makeShadow(checkbox.label, 0, 0, 0, 1, -2, 2)
	return checkbox
end


-----------------------------------
-------- events ------------------
----------------------------------

KBBUI.OnBuffColorSliderChange = function(slider)

	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local r, g, b = group:getBuffColor()

	if slider == uiWin.buffColorGroup.rSlider then
		r = slider:GetPosition()
	elseif slider == uiWin.buffColorGroup.gSlider then
		g = slider:GetPosition()
	elseif slider == uiWin.buffColorGroup.bSlider then
		b = slider:GetPosition()
	end

	group:setBuffColor(r, g, b)
	KBBUI.UpdateBuffColorPreview(group, uiWin)
end

KBBUI.OnDebuffColorSliderChange = function(slider)

	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local r, g, b = group:getDebuffColor()

	if slider == uiWin.debuffColorGroup.rSlider then
		r = slider:GetPosition()
	elseif slider == uiWin.debuffColorGroup.gSlider then
		g = slider:GetPosition()
	elseif slider == uiWin.debuffColorGroup.bSlider then
		b = slider:GetPosition()
	end

	group:setDebuffColor(r, g, b)
	KBBUI.UpdateDebuffColorPreview(group, uiWin)
end

KBBUI.OnFontColorSliderChange = function (slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local r, g, b = group:getFontColor()

	if slider == uiWin.fontColorGroup.rSlider then
		r = slider:GetPosition()
	elseif slider == uiWin.fontColorGroup.gSlider then
		g = slider:GetPosition()
	elseif slider == uiWin.fontColorGroup.bSlider then
		b = slider:GetPosition()
	end

	group:setFontColor(r, g, b)
	KBBUI.UpdateFontColorPreview(group, uiWin)
end

KBBUI.OnDebuffFontColorSliderChange = function(slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local r, g, b = group:getDebuffFontColor()

	if slider == uiWin.debuffFontColorGroup.rSlider then
		r = slider:GetPosition()
	elseif slider == uiWin.debuffFontColorGroup.gSlider then
		g = slider:GetPosition()
	elseif slider == uiWin.debuffFontColorGroup.bSlider then
		b = slider:GetPosition()
	end

	group:setDebuffFontColor(r, g, b)
	KBBUI.UpdateFontColorPreview(group, uiWin)
end
KBBUI.OnBGASliderChange = function(slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local alpha = uiWin.bgaSlider:GetPosition()
	group:setBackgroundAlpha(alpha)
	KBBUI.UpdateBGColorPreview(group, uiWin)
end

KBBUI.OnBGColorSliderChange = function(slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local r, g, b = group:getBackgroundColor()

	if slider == uiWin.bgColorGroup.rSlider then
		r = slider:GetPosition()
	elseif slider == uiWin.bgColorGroup.gSlider then
		g = slider:GetPosition()
	elseif slider == uiWin.bgColorGroup.bSlider then
		b = slider:GetPosition()
	end

	group:setBackgroundColor(r, g, b)
	KBBUI.UpdateBGColorPreview(group, uiWin)
end

KBBUI.OnBGColorCheckboxChange = function(checkbox)
	if checkbox.nochange then return end
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local uiWin = KBB.uiWindow

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end

	group.settings.backgroundUsesSameColor = checkbox:GetChecked()
	uiWin.bgColorGroup:SetEnabled(not group.settings.backgroundUsesSameColor)
	group:updateColors()
	KBBUI.UpdateBGColorPreview(group, uiWin)
end

KBBUI.OnFontSizeSliderChange = function(slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local _, group = KBB.GroupWithId(groupID)
	if not group then return end

	local fs = slider:GetPosition()
	group:setFontSize(fs)
end

KBBUI.OnHeightSliderChange = function(slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	group:setBarHeight(slider:GetPosition())
end

KBBUI.OnWidthSliderChange = function(slider)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	group:setBarWidth(slider:GetPosition())
end

KBBUI.OnXSliderChange = function(slider)
	if slider.nochange then return end
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local x = slider:GetPosition()
	local y = group.settings.position.y
	group:move(x, y)
end

KBBUI.OnYSliderChange = function(slider)
	if slider.nochange then return end
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local y = slider:GetPosition()
	local x = group.settings.position.x
	group:move(x, y)
end

KBBUI.OnWatchTypeRadioGroupChange = function(radioGroup)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local selected = radioGroup:GetSelectedIndex()
	local arg = "debuff"
	if (selected == 1) then
		arg = "buff"
	end
	group:setWatchType(arg)
	KBB.RescanGroup(group)
end

KBBUI.OnFilterTypeRadioGroupChange = function(radioGroup)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local selected = radioGroup:GetSelectedIndex()
	local arg = "blacklist"
	if (selected == 1) then
		arg = "whitelist"
	end
	group:setFilterType(arg)
	KBB.RescanGroup(group)
end

KBBUI.OnUnitRadioGroupChange = function(radioGroup)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local selected = radioGroup:GetSelectedIndex()
	local arg
	if (selected == 1) then
		arg = "player"
	elseif (selected == 2) then
		arg = "player.target"
	elseif (selected == 3) then
		arg = "focus"
	else
		arg = "player.pet"
	end
	group:setUnit(arg)
	KBB.RescanGroup(group)
end

KBBUI.OnSortRadioGroupChange = function(radioGroup)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local selected = radioGroup:GetSelectedIndex()
	local arg = "nosort"
	if (selected == 1) then
		arg = "asc"
	elseif (selected == 2) then
		arg = "desc"
	end
	local flag
	if (arg == "asc" or arg == "desc") then
		flag = true
		group.settings.sortOrder = arg
	end
	group.settings.sorts = flag or false
	if (flag) then
		group:sortBars()
		group:alignBars()
	end
end

KBBUI.OnGrowthRadioGroupChange = function(radioGroup)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	local selected = radioGroup:GetSelectedIndex()
	local arg = "down"
	if (selected == 1) then
		arg = "up"
	end
	group.settings.growth = arg
	if (group.settings.sorts) then group:sortBars() end
	group:alignBars()
end

KBBUI.OnCancelCheckboxChange = function(checkbox)
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	group.settings.cancelClick = checkbox:GetChecked()
end

KBBUI.OnAddGroup = function(button)
	KBB.AddGroup()
	local index = KBBUI.LoadGroupList(KBB.uiWindow:GetCurrentChar())
	KBB.uiWindow.groupsList:SetSelectedIndex(index)
end
KBBUI.OnRenameGroup = function(button)
	KBB.MakeGroupRenameWindow()

	local profile = KBB.uiWindow:GetCurrentChar()
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	KBB.uiRenameWindow.groupID = groupID
	KBB.uiRenameWindow.profile = profile
	KBB.uiRenameWindow.textEdit:SetText("")

	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	local oldName = group.settings.name
	KBB.uiRenameWindow.oldName:SetText(oldName)
	KBB.uiRenameWindow.oldNameShadow:SetText(oldName)
	KBB.uiRenameWindow:SetVisible(true)
end
KBBUI.OnRemoveGroup = function(button)
	local selected = KBB.uiWindow.groupsList:GetSelectedIndex()
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	KBB.RemoveGroupAtIndex(i)
	local n = KBBUI.LoadGroupList(KBB.uiWindow:GetCurrentChar())
	KBB.uiWindow.groupsList:SetSelectedIndex(math.min(selected, n))
end
KBBUI.OnToggleAnchors = function(button)
	KBB.ToggleAnchors()
end

KBBUI.OnConfirmFilterClick = function(button)
	if not button.contextInfo.mode then return end
	local mode = button.contextInfo.mode
	local filter = KBB.uiWindow.filterTextfield:GetText()
	if string.len(filter) > 0 then

		local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
		local i, group = KBB.GroupWithId(groupID)
		if not group then return end
		if (mode == "add") then
			group:addFilter(filter)
		elseif (mode == "edit") then
--			local index = button.contextInfo.value
--			local selectedFilterIndex = KBB.uiWindow.filtersList:GetSelectedValue()
			local index = KBB.uiWindow.filtersList:GetSelectedIndex()
			group:removeFilter(index)
			group:addFilterAtIndex(filter, index)
		end
		KBB.uiWindow.filterTextfield:SetText("")
		KBB.RescanGroup(group)
		KBBUI.LoadGroupFilters(group, KBB.uiWindow)
	end
	button.contextInfo.mode = false

	KBB.uiWindow.filterModeLabel:SetText(FILTER_MODE_NONE)
	KBB.uiWindow.filterModeLabel.KLHshadow:SetText(FILTER_MODE_NONE)
	KBB.uiWindow.filterInputArea:SetVisible(false)
	KBB.uiWindow.filterTextfield:SetKeyFocus(false)
end

KBBUI.OnAddFilter = function(button)
--	local filter = KBB.uiWindow.filterTextfield:GetText()
--	if string.len(filter) > 0 then
--		local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
--		local i, group = KBB.GroupWithId(groupID)
--		group:addFilter(filter)
--
--		KBB.uiWindow.filterTextfield:SetText("")
--		KBB.uiWindow.filterTextfield:SetKeyFocus(false)
--		KBB.RescanGroup(group)
--		KBBUI.LoadGroupFilters(group, KBB.uiWindow)
--	end
	KBB.uiWindow.filterInputArea:SetVisible(true)
	KBB.uiWindow.filterConfirmButton.contextInfo = { mode = "add" }
	KBB.uiWindow.filterModeLabel:SetText(FILTER_MODE_ADD)
	KBB.uiWindow.filterModeLabel.KLHshadow:SetText(FILTER_MODE_ADD)
	KBB.uiWindow.filterTextfield:SetText("")
	KBB.uiWindow.filterTextfield:SetKeyFocus(true)
	KBB.uiWindow.filterTextfield:SetCursor(0)
end

KBBUI.OnRemoveFilter = function(button)
	local selectedFilterIndex = KBB.uiWindow.filtersList:GetSelectedValue()
	local index = KBB.uiWindow.filtersList:GetSelectedIndex()
	local groupID = KBB.uiWindow.groupsList:GetSelectedValue()
	if groupID == nil then return end
	local i, group = KBB.GroupWithId(groupID)
	if not group then return end
	group:removeFilter(selectedFilterIndex)
	KBB.RescanGroup(group)
	local fcount = KBBUI.LoadGroupFilters(group, KBB.uiWindow)
	KBB.uiWindow.filtersList:SetSelectedIndex(math.min(index, fcount))
	KBBUI.SetFilterButtonsState()

end
KBBUI.OnEditFilter = function(button)
	KBB.uiWindow.filterInputArea:SetVisible(true)
	local selectedFilterIndex = KBB.uiWindow.filtersList:GetSelectedValue()
	local filterString = KBB.uiWindow.filtersList:GetSelectedItem()
	KBB.uiWindow.filterConfirmButton.contextInfo = { mode = "edit", value = selectedFilterIndex }
	KBB.uiWindow.filterConfirmButton:SetEnabled(true)
	KBB.uiWindow.filterModeLabel:SetText(FILTER_MODE_EDIT)
	KBB.uiWindow.filterModeLabel.KLHshadow:SetText(FILTER_MODE_EDIT)
	KBB.uiWindow.filterTextfield:SetText(filterString)
	KBB.uiWindow.filterTextfield:SetKeyFocus(true)
	KBB.uiWindow.filterTextfield:SetCursor(string.len(filterString))
end

KBBUI.OnFilterSelect = function(list)
	KBBUI.SetFilterButtonsState()
end

KBBUI.OnMinimapCheckboxChange = function(checkbox)
	if checkbox.nochange then return end

	KBB.characterDB.settings.minimapButton = checkbox:GetChecked()

-- 	local mmVisible = UI.Native.MapMini:GetLoaded()
-- 	KBB.minimapButton:SetVisible(mmVisible and KBB.characterDB.settings.minimapButton)
	KBB.minimapButton:SetVisible(KBB.characterDB.settings.minimapButton)
	KBB.uiWindow.minimapSlider:SetEnabled(KBB.characterDB.settings.minimapButton)
	KBB.uiWindow.minimapOffsetSlider:SetEnabled(KBB.characterDB.settings.minimapButton)
end

KBBUI.OnMinimapSliderChange = function(slider)
	KBB.characterDB.settings.minimapButtonAngle = slider:GetPosition()
	Library.KLH.attachButtonToMinimap(KBB.minimapButton, KBB.characterDB.settings.minimapButtonAngle, KBB.characterDB.settings.minimapButtonOffset)
end

KBBUI.OnMinimapOffsetSliderChange = function(slider)
	KBB.characterDB.settings.minimapButtonOffset = slider:GetPosition()
	Library.KLH.attachButtonToMinimap(KBB.minimapButton, KBB.characterDB.settings.minimapButtonAngle, KBB.characterDB.settings.minimapButtonOffset)
end
KBBUI.OnMinimapButtonSizeSliderChange = function(slider)
	local p = slider:GetPosition() 
	KBB.characterDB.settings.minimapButtonSize = p	
	KBB.minimapButton:SetWidth(p)
	KBB.minimapButton:SetHeight(p)
	Library.KLH.attachButtonToMinimap(KBB.minimapButton, KBB.characterDB.settings.minimapButtonAngle, KBB.characterDB.settings.minimapButtonOffset)
end
KBBUI.OnNativeTooltipCheckboxChange = function(checkbox)
	if checkbox.nochange then return end

	KBB.characterDB.settings.useNativeTooltip = checkbox:GetChecked()
end

KBBUI.OnEnabledCheckboxChange = function(checkbox)
	if checkbox.nochange then return end

	KBB.characterDB.settings.on = checkbox:GetChecked()
	KBB.setEnabled(KBB.characterDB.settings.on)
end

KBBUI.OnExportProfileClick = function(button)
	local uiWin = KBB.uiWindow

	local destName = uiWin.exportProfileTextfield:GetText()
	if string.len(destName) > 0 then
		local thisChar = uiWin:GetCurrentChar()
		local result = KBB.exportGroups(thisChar, destName)
		if (result) then
			print(KBBLS("KBB_PROFILE_EXPORT_SUCCESS", thisChar, destName))
		else
			print(KBBLS("KBB_PROFILE_ALREADY_EXISTS", destName))
		end
	end
	KBBUI.LoadProfilesList()
end

KBBUI.OnImportProfileClick = function(button)
	local uiWin = KBB.uiWindow
	local sourceChar = uiWin.profilesList:GetSelectedItem()

	if not sourceChar then return end
	local thisChar = uiWin:GetCurrentChar()
	KBB.importGroups(sourceChar, thisChar, true)
	KBBUI.LoadGroupList(thisChar)
	KBB.uiWindow:SelectGroupWithIndex(1)
end

KBBUI.OnDeleteProfileClick = function(button)
	local uiWin = KBB.uiWindow
	local sourceChar = uiWin.profilesList:GetSelectedItem()

	if not sourceChar then return end
	local thisChar = uiWin:GetCurrentChar()
	KBB.deleteProfile(sourceChar)
	KBBUI.LoadProfilesList()
end
--------------------------------------------------------------
----------------------make UI window -------------------------
--------------------------------------------------------------
function KBBUI.OnClose()
	KBB.uiWindow.filterTextfield:SetText("")
	Library.KLH.makeChildrenPerformSelector(KBB.uiWindow, true, "SetKeyFocus", false)
	if KBB.uiRenameWindow then
		KBB.uiRenameWindow.textEdit:SetKeyFocus(false)
		KBB.uiRenameWindow:SetVisible(false)
	end
end

function KBB.MakeGroupRenameWindow()
	if (KBB.uiRenameWindow) then return end
     Command.System.Watchdog.Quiet()
     
	KBB.uiRenameWindow = Library.LibSimpleWidgets.Window("KBB.uiRenameWindow", KBB.uiContext)
	local uiRWin = KBB.uiRenameWindow
	uiRWin:SetLayer(5)

	uiRWin:SetTitle(KBB.localString("KBB_SETTINGS_GROUP_RENAME_WINDOW_TITLE"))
	uiRWin:SetPoint("CENTER", KBB.uiContext, "CENTER", 0, 0)
	uiRWin:SetWidth(RENAME_WINDOW_WIDTH)
	uiRWin:SetHeight(RENAME_WINDOW_HEIGHT)

	uiRWin.textEdit = UI.CreateFrame("RiftTextfield", uiRWin:GetName() .. ".textEdit", uiRWin)
	uiRWin.textEdit:SetWidth(200)
	uiRWin.textEdit:SetBackgroundColor(0, 0, 0, EDITFIELD_ALPHA)
	uiRWin.textEdit:SetText("")
	uiRWin.textEdit:SetPoint("TOPCENTER", uiRWin:GetContent(), "TOPCENTER", 0, 20)

	local oldNameLabel = KBBUI.MakeLabel(uiRWin:GetName() .. ".oldNameLabel", uiRWin, KBBLS("UI_OLD_GROUP_NAME_LABEL_TEXT"))
	oldNameLabel:SetPoint("BOTTOMLEFT", uiRWin.textEdit, "TOPLEFT", 30, -5)

	uiRWin.oldName = UI.CreateFrame("Text", uiRWin:GetName() .. ".oldName", uiRWin)
	uiRWin.oldName:SetFontSize(16)
	uiRWin.oldName:SetPoint("BOTTOMLEFT", oldNameLabel, "BOTTOMRIGHT", 5, 0)
	uiRWin.oldNameShadow = Library.KLH.makeShadow(uiRWin.oldName, 0, 0, 0, 1, -2, -2)

	uiRWin.okButton = UI.CreateFrame("RiftButton", uiRWin:GetName() .. ".okButton", uiRWin)
	uiRWin.okButton:SetText("OK")
	uiRWin.okButton:SetPoint("BOTTOMRIGHT", uiRWin:GetContent(), "BOTTOMRIGHT")
	uiRWin.okButton.Event.LeftPress = function()
		local name = uiRWin.textEdit:GetText()
		if string.len(name) > 0 then
			local _, group = KBB.GroupWithId(uiRWin.groupID)
			if not group then return end
			group:setName(name)
			local selected = KBB.uiWindow.groupsList:GetSelectedIndex()
			KBBUI.LoadGroupList(uiRWin.profile)
			KBB.uiWindow.groupsList:SetSelectedIndex(selected)
			-- 					uiRWin.textEdit:SetKeyFocus(false)
		end
		uiRWin.textEdit:SetKeyFocus(false)
		uiRWin:SetVisible(false)
	end

	uiRWin.backButton = UI.CreateFrame("RiftButton", uiRWin:GetName() .. ".backButton", uiRWin)
	uiRWin.backButton:SetText(KBBLS("UI_CANCEL_BUTTON_TEXT"))
	uiRWin.backButton:SetPoint("BOTTOMRIGHT", uiRWin.okButton, "BOTTOMLEFT")
	uiRWin.backButton.Event.LeftPress = function()
		uiRWin.textEdit:SetKeyFocus(false)
		uiRWin:SetVisible(false)
	end
	uiRWin:SetVisible(false)

end

function KBB.MakeUI()
	if (KBB.uiWindow) then return end

    Command.System.Watchdog.Quiet()
	----------- make window   ------------------

	KBB.uiWindow = Library.LibSimpleWidgets.Window("KBB.uiWindow", KBB.uiContext)
	local uiWin = KBB.uiWindow
	local winCon = uiWin:GetContent()
	uiWin:SetVisible(false)

	uiWin:SetTitle(KBB.localString("KBB_SETTINGS_WINDOW_TITLE"))
	uiWin:SetPoint("CENTER", KBB.uiContext, "CENTER", 0, 0)
	uiWin:SetWidth(WINDOW_WIDTH)
	uiWin:SetHeight(WINDOW_HEIGHT)

--	Library.KLH.addCloseButton(uiWin, function() KBBUI.OnClose() end)
	uiWin:SetCloseButtonVisible(true)
	uiWin.Event.Close = OnClose

	------------- make top row controls -------------------
	uiWin.minimapCheckbox = KBBUI.MakeCheckbox(uiWin:GetName() .. ".minimapCheckbox", winCon, KBBLS("UI_MINIMAP_CHECKBOX_TEXT"))
	--  Library.LibSimpleWidgets.Checkbox(uiWin:GetName() .. ".minimapCheckbox", uiWin)
	--  uiWin.minimapCheckbox:SetText(KBBLS("UI_MINIMAP_CHECKBOX_TEXT"))
	uiWin.minimapCheckbox:SetPoint("TOPLEFT", winCon, "TOPLEFT", 20, 0)
	uiWin.minimapCheckbox.Event.CheckboxChange = KBBUI.OnMinimapCheckboxChange

    uiWin.enabledCheckbox = KBBUI.MakeCheckbox(uiWin:GetName() .. ".enabledCheckbox", winCon, KBBLS("UI_ENABLED_CHECKBOX_TEXT"))
	--  Library.LibSimpleWidgets.Checkbox(uiWin:GetName() .. ".enabledCheckbox", uiWin)
	--  uiWin.enabledCheckbox:SetText(KBBLS("UI_ENABLED_CHECKBOX_TEXT"))
	uiWin.enabledCheckbox:SetPoint("RIGHT", winCon, "RIGHT", -5, nil)
	uiWin.enabledCheckbox:SetPoint("TOP", uiWin.minimapCheckbox, "TOP")
	uiWin.enabledCheckbox.Event.CheckboxChange = KBBUI.OnEnabledCheckboxChange
	
	uiWin.anchorsButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".anchorsButton", winCon)
	uiWin.anchorsButton:SetText(KBBLS("UI_TOGGLE_ANCHORS_BUTTON_TEXT"))
	uiWin.anchorsButton:SetPoint("TOPRIGHT", uiWin.enabledCheckbox, "BOTTOMRIGHT", 0, 0)
	uiWin.anchorsButton.Event.LeftClick = KBBUI.OnToggleAnchors

	uiWin.nativeTooltipCheckbox = KBBUI.MakeCheckbox(uiWin:GetName() .. ".nativeTooltipCheckbox", winCon, KBBLS("UI_NATIVE_TOOLTIP_CHECKBOX_TEXT"))
	uiWin.nativeTooltipCheckbox:SetPoint("RIGHTCENTER", uiWin.anchorsButton, "LEFTCENTER", -15, 0)
	uiWin.nativeTooltipCheckbox.Event.CheckboxChange = KBBUI.OnNativeTooltipCheckboxChange

	local minimapAngleLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".minimapAngleLabel", winCon, KBBLS("UI_MINIMAP_BUTTON_ANGLE_LABEL"))
	minimapAngleLabel:SetPoint("LEFTCENTER", uiWin.minimapCheckbox, "RIGHTCENTER", 15, 0)
-- 	minimapAngleLabel:SetPoint("CENTERY", uiWin.anchorsButton, "CENTERY")  	
	
	uiWin.minimapSlider = KBBUI.MakeSlider(uiWin:GetName() .. ".minimapSlider", winCon, 0, 360, 185)
	uiWin.minimapSlider:SetPoint("LEFTCENTER", minimapAngleLabel, "RIGHTCENTER", 15, 0)
	uiWin.minimapSlider.Event.SliderChange = KBBUI.OnMinimapSliderChange

	local minimapOffsetLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".minimapOffsetLabel", winCon, KBBLS("UI_MINIMAP_BUTTON_OFFSET_LABEL") )
	minimapOffsetLabel:SetPoint("RIGHT", minimapAngleLabel, "RIGHT")
	minimapOffsetLabel:SetPoint("CENTERY", uiWin.anchorsButton, "CENTERY")
	
    uiWin.minimapOffsetSlider = KBBUI.MakeSlider(uiWin:GetName() .. ".minimapOffsetSlider", winCon, -100, 100, 185)
	uiWin.minimapOffsetSlider:SetPoint("LEFTCENTER", minimapOffsetLabel, "RIGHTCENTER", 15, 0)
	uiWin.minimapOffsetSlider.Event.SliderChange = KBBUI.OnMinimapOffsetSliderChange
	
	local minimapButtonSizeLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".minimapButtonSizeLabel", winCon, KBBLS("UI_MINIMAP_BUTTON_SIZE_LABEL") )
	minimapButtonSizeLabel:SetPoint("LEFTCENTER", uiWin.minimapSlider, "RIGHTCENTER", 15, 0)
	
    uiWin.minimapButtonSizeSlider = KBBUI.MakeSlider(uiWin:GetName() .. ".minimapButtonSizeSlider", winCon, 10, 100, 185)
	uiWin.minimapButtonSizeSlider:SetPoint("LEFTCENTER", minimapButtonSizeLabel, "RIGHTCENTER", 15, 0)
	uiWin.minimapButtonSizeSlider.Event.SliderChange = KBBUI.OnMinimapButtonSizeSliderChange
	
	-----------  make tab view   ------------------ 	
	
	
	uiWin.tabView = Library.LibSimpleWidgets.TabView(uiWin:GetName()..".tabView", winCon)
	uiWin.tabView:SetTabPosition("top")
	uiWin.tabView:SetActiveFontColor(ACTIVE_FONT_R, ACTIVE_FONT_G, ACTIVE_FONT_B, 1)
	uiWin.tabView:SetHighlightFontColor(HIGHLIGHT_TAB_FONT_R, HIGHLIGHT_TAB_FONT_G, HIGHLIGHT_TAB_FONT_B, 1)
	uiWin.tabView:SetFontSize(UI_FONT_SIZE)

    local separator1 = UI.CreateFrame("Frame", uiWin:GetName()..".tabView", winCon)
	separator1:SetHeight(2)
	separator1:SetBackgroundColor(BORDER_R, BORDER_G, BORDER_B, 1)
	separator1:SetPoint("BOTTOMLEFT", uiWin.tabView, "TOPLEFT", 0, -3)
	separator1:SetPoint("BOTTOMRIGHT", uiWin.tabView, "TOPRIGHT", 0, -3)
	
	--   local winCon = uiWin:GetContent()
	uiWin.tabView:SetPoint("TOP", uiWin.anchorsButton, "BOTTOM", nil, 5)
	uiWin.tabView:SetPoint("LEFT", winCon, "LEFT")
	uiWin.tabView:SetPoint("BOTTOMRIGHT", winCon, "BOTTOMRIGHT")
	uiWin.tabView.tabContent:SetBackgroundColor(0,0,0,0)

	-----------  make groups tab   ------------------
	uiWin.groupsTab = UI.CreateFrame("Frame", uiWin:GetName()..".groupsTab", uiWin.tabView.tabContent)
	uiWin.tabView:AddTab(KBBLS("UI_GROUPS_TAB_LABEL"),  uiWin.groupsTab)

	-----------  make profiles tab   ------------------
	uiWin.profilesTab = UI.CreateFrame("Frame", uiWin:GetName()..".profilesTab", uiWin.tabView.tabContent)
	uiWin.tabView:AddTab(KBBLS("UI_PROFILES_TAB_LABEL"),  uiWin.profilesTab)

	------------------------- groups list ------------
	local groupsScrollView = Library.LibSimpleWidgets.ScrollView(uiWin:GetName() .. ".groupsScrollView", uiWin.groupsTab)
	groupsScrollView:SetWidth(150)
	groupsScrollView:SetBackgroundColor(0, 0, 0, 0)
	groupsScrollView:SetPoint("TOPLEFT", uiWin.groupsTab, "TOPLEFT", 10, 30)
	groupsScrollView:SetPoint("BOTTOMLEFT", uiWin.groupsTab, "BOTTOMLEFT", 10, -10)
	groupsScrollView:SetBorder(2, BORDER_R, BORDER_G, BORDER_B, 1)

	local groupsLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".groupsLabel", uiWin.groupsTab, KBBLS("UI_GROUPS_LABEL_TEXT"))
	groupsLabel:SetPoint("BOTTOMLEFT", groupsScrollView, "TOPLEFT", 0, -3)

	uiWin.groupsList = Library.LibSimpleWidgets.List(uiWin:GetName()..".groupsList", uiWin.groupsTab)
	uiWin.groupsList:SetBackgroundColor(0, 0, 0, 0)
	uiWin.groupsList:SetFontSize(UI_FONT_SIZE)
	groupsScrollView:SetContent(uiWin.groupsList)
	uiWin.groupsList.Event.ItemSelect = KBBUI.OnGroupSelect

	------------------------- add/remove/rename  buttons ------------

	uiWin.renameButton = UI.CreateFrame("RiftButton", uiWin:GetName()..".renameButton", uiWin.groupsTab)
	uiWin.renameButton:SetText(KBBLS("UI_RENAME_GROUP_BUTTON_TEXT"))
	uiWin.renameButton:SetPoint("LEFT", groupsScrollView, "RIGHT", 20, nil)
	uiWin.renameButton:SetPoint("TOP", groupsLabel, "TOP")
	uiWin.renameButton.Event.LeftClick = KBBUI.OnRenameGroup


	uiWin.addButton = UI.CreateFrame("RiftButton", uiWin:GetName()..".addButton", uiWin.groupsTab)
	uiWin.addButton:SetText(KBBLS("UI_ADD_GROUP_BUTTON_TEXT"))
	uiWin.addButton:SetPoint("TOPLEFT", uiWin.renameButton, "TOPRIGHT", 40, 0)
	uiWin.addButton.Event.LeftClick = KBBUI.OnAddGroup

	uiWin.removeButton = UI.CreateFrame("RiftButton", uiWin:GetName()..".removeButton", uiWin.groupsTab)
	uiWin.removeButton:SetText(KBBLS("UI_REMOVE_GROUP_BUTTON_TEXT"))
	uiWin.removeButton:SetPoint("TOPLEFT", uiWin.addButton, "TOPRIGHT", 0, 0)
	uiWin.removeButton.Event.LeftClick = KBBUI.OnRemoveGroup

	-------------------------  group settings tab view ----------------
	uiWin.groupSettingsTabView = Library.LibSimpleWidgets.TabView(uiWin:GetName()..".groupSettingsTabView", uiWin.groupsTab)
	uiWin.groupSettingsTabView:SetTabPosition("top")
	uiWin.groupSettingsTabView:SetActiveFontColor(ACTIVE_FONT_R, ACTIVE_FONT_G, ACTIVE_FONT_B, 1)
	uiWin.groupSettingsTabView:SetHighlightFontColor(HIGHLIGHT_TAB_FONT_R, HIGHLIGHT_TAB_FONT_G, HIGHLIGHT_TAB_FONT_B, 1)
	uiWin.groupSettingsTabView:SetFontSize(UI_FONT_SIZE)
	--   local winCon = uiWin:GetContent()
	uiWin.groupSettingsTabView:SetPoint("LEFT", uiWin.renameButton, "LEFT")
	uiWin.groupSettingsTabView:SetPoint("TOP", uiWin.renameButton, "BOTTOM", nil, 5)
	uiWin.groupSettingsTabView:SetPoint("BOTTOMRIGHT", uiWin.groupsTab, "BOTTOMRIGHT", -10, 0)
	uiWin.groupSettingsTabView.tabContent:SetBackgroundColor(0,0,0,0)

	-------------------------- group settings tabs ---------------------

	uiWin.generalTab = UI.CreateFrame("Frame", uiWin:GetName()..".generalTab", uiWin.groupSettingsTabView.tabContent)
	uiWin.groupSettingsTabView:AddTab(KBBLS("UI_GENERAL_TAB_LABEL"),  uiWin.generalTab)

	uiWin.filtersTab = UI.CreateFrame("Frame", uiWin:GetName()..".filtersTab", uiWin.groupSettingsTabView.tabContent)
	uiWin.groupSettingsTabView:AddTab(KBBLS("UI_FILTERS_TAB_LABEL"),  uiWin.filtersTab)

	uiWin.lookTab = UI.CreateFrame("Frame", uiWin:GetName()..".lookTab", uiWin.groupSettingsTabView.tabContent)
	uiWin.groupSettingsTabView:AddTab(KBBLS("UI_LOOK_TAB_LABEL"),  uiWin.lookTab)

	----------------------------------------------------------------------
	-------------------------- general tab -------------------------------
	local tabWidth = uiWin.generalTab:GetWidth()

	-------------------------- group position -----------------------------------
	local positionLabel = KBBUI.MakeLabel( uiWin:GetName()..".positionLabel", uiWin.generalTab, KBBLS("UI_POSITION_LABEL_TEXT"))
	positionLabel:SetPoint("TOPLEFT", uiWin.generalTab, "TOPLEFT", 10 + tabWidth/2, 30)

	local xposLabel =  KBBUI.MakeLabel( uiWin:GetName()..".xposLabel", uiWin.generalTab, KBBLS("UI_XPOS_LABEL_TEXT"))
	xposLabel:SetPoint("TOPLEFT", positionLabel, "BOTTOMLEFT", 0, 3)

	uiWin.xSlider = KBBUI.MakeSlider(uiWin:GetName()..".xSlider", uiWin.generalTab, -1000, 1000, 185)
	uiWin.xSlider:SetPoint("CENTERLEFT", xposLabel, "CENTERRIGHT", 8, 0)
	uiWin.xSlider.Event.SliderChange = KBBUI.OnXSliderChange


	local yposLabel =  KBBUI.MakeLabel( uiWin:GetName()..".yposLabel", uiWin.generalTab, KBBLS("UI_YPOS_LABEL_TEXT"))
	yposLabel:SetPoint("TOPLEFT", xposLabel, "BOTTOMLEFT", 0, 3)

	uiWin.ySlider = KBBUI.MakeSlider(uiWin:GetName()..".ySlider", uiWin.generalTab, -1000, 1000, 185)
	uiWin.ySlider:SetPoint("CENTERLEFT", yposLabel, "CENTERRIGHT", 8, 0)
	uiWin.ySlider.Event.SliderChange = KBBUI.OnYSliderChange

	-------------------------- bar size --------------------------------

	local barSizeLabel = KBBUI.MakeLabel( uiWin:GetName()..".positionLabel", uiWin.generalTab, KBBLS("UI_BAR_SIZE_LABEL_TEXT"))
	barSizeLabel:SetPoint("TOPLEFT", uiWin.generalTab, "TOPLEFT", 10, 30)

	local widthLabel =  KBBUI.MakeLabel( uiWin:GetName()..".widthLabel", uiWin.generalTab, KBBLS("UI_WIDTH_LABEL_TEXT"))
	widthLabel:SetPoint("TOPLEFT", barSizeLabel, "BOTTOMLEFT", 0, 3)

	uiWin.widthSlider = KBBUI.MakeSlider(uiWin:GetName()..".widthSlider", uiWin.generalTab, 10, MAX_BAR_WIDTH, 185, 10)
	uiWin.widthSlider:SetPoint("CENTERLEFT", widthLabel, "CENTERRIGHT", 8, 0)
	uiWin.widthSlider.Event.SliderChange = KBBUI.OnWidthSliderChange


	local heightLabel =  KBBUI.MakeLabel( uiWin:GetName()..".heightLabel", uiWin.generalTab, KBBLS("UI_HEIGHT_LABEL_TEXT"))
	heightLabel:SetPoint("TOPLEFT", widthLabel, "BOTTOMLEFT", 0, 3)

	uiWin.heightSlider = KBBUI.MakeSlider(uiWin:GetName()..".heightSlider", uiWin.generalTab, 10, MAX_BAR_HEIGHT, 185, 10)
	uiWin.heightSlider:SetPoint("LEFT", uiWin.widthSlider, "LEFT")
	uiWin.heightSlider:SetPoint("BOTTOM", heightLabel, "BOTTOM")
	uiWin.heightSlider.Event.SliderChange = KBBUI.OnHeightSliderChange


	--------------------------- unit radios ---------------------------

	local unitLabel = KBBUI.MakeLabel( uiWin:GetName()..".unitLabel", uiWin.generalTab, KBBLS("UI_UNIT_LABEL_TEXT"))
	unitLabel:SetPoint("TOPLEFT", heightLabel, "BOTTOMLEFT", 0, 3)

	uiWin.unitRadios = Library.LibSimpleWidgets.RadioButtonGroup(uiWin:GetName()..".unitRadios")
	uiWin.unitRadios.Event.RadioButtonChange = KBBUI.OnUnitRadioGroupChange

	local unitRadio1 = KBBUI.MakeRadioButton(uiWin:GetName()..".playerRadio", uiWin.generalTab, KBBLS("UI_PLAYER_RADIO_TEXT"))
	unitRadio1:SetPoint("TOPLEFT", unitLabel, "BOTTOMLEFT", 0, 3)
	uiWin.unitRadios:AddRadioButton(unitRadio1)

	local unitRadio2 = KBBUI.MakeRadioButton(uiWin:GetName()..".playerTargetRadio", uiWin.generalTab, KBBLS("UI_PLAYER_TARGET_RADIO_TEXT"))
	unitRadio2:SetPoint("BOTTOMLEFT", unitRadio1, "BOTTOMLEFT", 100, 0)
	uiWin.unitRadios:AddRadioButton(unitRadio2)

	local unitRadio3 = KBBUI.MakeRadioButton(uiWin:GetName()..".focusRadio", uiWin.generalTab, KBBLS("UI_FOCUS_RADIO_TEXT"))
	unitRadio3:SetPoint("TOPLEFT", unitRadio1, "BOTTOMLEFT", 0, 3)
	uiWin.unitRadios:AddRadioButton(unitRadio3)

	local unitRadio4 = KBBUI.MakeRadioButton(uiWin:GetName()..".playerPetRadio", uiWin.generalTab, KBBLS("UI_PLAYER_PET_RADIO_TEXT"))
	unitRadio4:SetPoint("TOPLEFT", unitRadio2, "BOTTOMLEFT", 0, 3)
	uiWin.unitRadios:AddRadioButton(unitRadio4)

	--------------------------  watching type radios ------------------------

	local watchTypeLabel = KBBUI.MakeLabel(uiWin:GetName()..".watchTypeLabel", uiWin.generalTab, KBBLS("UI_WATCH_TYPE_LABEL_TEXT"))
	watchTypeLabel:SetPoint("TOPLEFT", unitRadio3, "BOTTOMLEFT", 0, 3)

	uiWin.watchTypeRadios = Library.LibSimpleWidgets.RadioButtonGroup(uiWin:GetName()..".watchTypeRadios")
	uiWin.watchTypeRadios.Event.RadioButtonChange = KBBUI.OnWatchTypeRadioGroupChange

	local watchRadio1 = KBBUI.MakeRadioButton(uiWin:GetName()..".buffRadio", uiWin.generalTab, KBBLS("UI_BUFFS_RADIO_TEXT"))
	watchRadio1:SetPoint("TOPLEFT", watchTypeLabel, "BOTTOMLEFT", 0, 3)
	uiWin.watchTypeRadios:AddRadioButton(watchRadio1)

	local watchRadio2 = KBBUI.MakeRadioButton(uiWin:GetName()..".debuffRadio", uiWin.generalTab, KBBLS("UI_DEBUFFS_RADIO_TEXT"))
	watchRadio2:SetPoint("BOTTOMLEFT", watchRadio1, "BOTTOMLEFT", 100, 0)
	uiWin.watchTypeRadios:AddRadioButton(watchRadio2)

	------------------------- filtering type radios------------

	local filterTypeLabel = KBBUI.MakeLabel(uiWin:GetName()..".filterTypeLabel", uiWin.generalTab, KBBLS("UI_FILTER_TYPE_LABEL_TEXT"))
	filterTypeLabel:SetPoint("TOPLEFT", watchRadio1, "BOTTOMLEFT", 0, 3)

	uiWin.filterTypeRadios = Library.LibSimpleWidgets.RadioButtonGroup(uiWin:GetName()..".filterTypeRadios")
	uiWin.filterTypeRadios.Event.RadioButtonChange = KBBUI.OnFilterTypeRadioGroupChange

	local filterRadio1 = KBBUI.MakeRadioButton(uiWin:GetName()..".whitelistRadio", uiWin.generalTab, KBBLS("UI_WHITELIST_RADIO_TEXT"))
	filterRadio1:SetPoint("TOPLEFT", filterTypeLabel, "BOTTOMLEFT", 0, 3)
	uiWin.filterTypeRadios:AddRadioButton(filterRadio1)

	local filterRadio2 = KBBUI.MakeRadioButton(uiWin:GetName()..".blacklistRadio", uiWin.generalTab, KBBLS("UI_BLACKLIST_RADIO_TEXT"))
	filterRadio2:SetPoint("BOTTOMLEFT", filterRadio1, "BOTTOMLEFT", 100, 0)
	uiWin.filterTypeRadios:AddRadioButton(filterRadio2)

	-------------------------- sort radios ----------------------
	uiWin.sortRadios =  Library.LibSimpleWidgets.RadioButtonGroup(uiWin:GetName()..".sortRadios")
	uiWin.sortRadios.Event.RadioButtonChange = KBBUI.OnSortRadioGroupChange

	local sortLabel =  KBBUI.MakeLabel(uiWin:GetName()..".sortLabel", uiWin.generalTab, KBBLS("UI_SORT_LABEL_TEXT"))
	sortLabel:SetPoint("TOPLEFT", yposLabel, "BOTTOMLEFT", 0, 3)

	local sortRadio1 = KBBUI.MakeRadioButton(uiWin:GetName()..".ascRadio", uiWin.generalTab, KBBLS("UI_SORT_ASCENDING_RADIO_TEXT"))
	sortRadio1:SetPoint("TOPLEFT", sortLabel, "BOTTOMLEFT", 0, 3)
	uiWin.sortRadios:AddRadioButton(sortRadio1)

	local sortRadio2 = KBBUI.MakeRadioButton(uiWin:GetName()..".descRadio", uiWin.generalTab, KBBLS("UI_SORT_DESCENDING_RADIO_TEXT"))
	sortRadio2:SetPoint("BOTTOMLEFT", sortRadio1, "BOTTOMLEFT", 100, 0)
	uiWin.sortRadios:AddRadioButton(sortRadio2)

	local sortRadio3 = KBBUI.MakeRadioButton(uiWin:GetName()..".nosortRadio", uiWin.generalTab, KBBLS("UI_NO_SORT_RADIO_TEXT"))
	sortRadio3:SetPoint("TOPLEFT", sortRadio1, "BOTTOMLEFT", 0, 3)
	uiWin.sortRadios:AddRadioButton(sortRadio3)

	-------------------------- growth radios ----------------------
	uiWin.growthRadios =  Library.LibSimpleWidgets.RadioButtonGroup(uiWin:GetName()..".growthRadios")
	uiWin.growthRadios.Event.RadioButtonChange = KBBUI.OnGrowthRadioGroupChange

	local growthLabel = KBBUI.MakeLabel( uiWin:GetName()..".growthLabel", uiWin.generalTab, KBBLS("UI_GROWTH_LABEL_TEXT"))
	growthLabel:SetPoint("TOPLEFT", sortRadio3, "BOTTOMLEFT", 0, 3)

	local growthRadio1 = KBBUI.MakeRadioButton(uiWin:GetName()..".growthUpRadio", uiWin.generalTab, KBBLS("UI_GROWTH_UP_RADIO_TEXT"))
	growthRadio1:SetPoint("TOPLEFT", growthLabel, "BOTTOMLEFT", 0, 3)
	uiWin.growthRadios:AddRadioButton(growthRadio1)

	local growthRadio2 = KBBUI.MakeRadioButton(uiWin:GetName()..".growthDownRadio", uiWin.generalTab, KBBLS("UI_GROWTH_DOWN_RADIO_TEXT"))
	growthRadio2:SetPoint("BOTTOMLEFT", growthRadio1, "BOTTOMLEFT", 100, 0)
	uiWin.growthRadios:AddRadioButton(growthRadio2)

	----------------------- right click cancel checkbox -------------

	uiWin.cancelCheckbox = KBBUI.MakeCheckbox(uiWin:GetName() .. ".cancelCheckbox", uiWin.generalTab, KBBLS("UI_RIGHT_CLICK_CANCEL_CHECKBOX_TEXT"))
	uiWin.cancelCheckbox:SetPoint("TOPLEFT", growthRadio1, "BOTTOMLEFT", 0, 30)
	uiWin.cancelCheckbox.Event.CheckboxChange = KBBUI.OnCancelCheckboxChange


	----------------------------------------------------------------------
	-------------------------- filters tab -------------------------------

	uiWin.filterInputArea = UI.CreateFrame("Frame", uiWin:GetName() .. ".filterInputArea", uiWin.filtersTab)

	uiWin.filterConfirmButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".filterConfirmButton", uiWin.filterInputArea)
	uiWin.filterConfirmButton:SetText(KBBLS("UI_OK_BUTTON_TEXT"))
	uiWin.filterConfirmButton:SetPoint("TOPRIGHT", uiWin.filtersTab, "TOPRIGHT", -10, 20)
	uiWin.filterConfirmButton.Event.LeftDown = KBBUI.OnConfirmFilterClick
	uiWin.filterConfirmButton.contextInfo = {}

	uiWin.filterTextfield = UI.CreateFrame("RiftTextfield", uiWin:GetName() .. ".filterTextfield", uiWin.filterInputArea)
	uiWin.filterTextfield:SetPoint("LEFT", uiWin.filtersTab, "LEFT", 10, nil)
	uiWin.filterTextfield:SetPoint("RIGHTCENTER", uiWin.filterConfirmButton, "LEFTCENTER", -10, 0)
	uiWin.filterTextfield:SetBackgroundColor(0, 0, 0, EDITFIELD_ALPHA)
	uiWin.filterTextfield:SetText("")

	uiWin.filterModeLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".filterModeLabel", uiWin.filtersTab, FILTER_MODE_NONE)
	uiWin.filterModeLabel:SetPoint("BOTTOMLEFT", uiWin.filterTextfield, "TOPLEFT", 0, -1)

	uiWin.addFilterButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".addFilterButton", uiWin.filtersTab)
	uiWin.addFilterButton:SetText(KBBLS("UI_ADD_FILTER_BUTTON_TEXT"))
	uiWin.addFilterButton:SetPoint("TOPLEFT", uiWin.filterConfirmButton, "BOTTOMLEFT", 0, 5)
	uiWin.addFilterButton.Event.LeftClick = KBBUI.OnAddFilter

	uiWin.editFilterButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".editFilterButton", uiWin.filtersTab)
	uiWin.editFilterButton:SetText(KBBLS("UI_EDIT_FILTER_BUTTON_TEXT"))
	uiWin.editFilterButton:SetPoint("TOPLEFT", uiWin.addFilterButton, "BOTTOMLEFT", 0, 5)
	uiWin.editFilterButton.Event.LeftClick = KBBUI.OnEditFilter

	uiWin.removeFilterButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".removeFilterButton", uiWin.filtersTab)
	uiWin.removeFilterButton:SetText(KBBLS("UI_REMOVE_FILTER_BUTTON_TEXT"))
	uiWin.removeFilterButton:SetPoint("TOPLEFT", uiWin.editFilterButton, "BOTTOMLEFT", 0, 5)
	uiWin.removeFilterButton.Event.LeftClick = KBBUI.OnRemoveFilter


	local filtersScrollView = Library.LibSimpleWidgets.ScrollView(uiWin:GetName() .. ".filtersScrollView", uiWin.filtersTab)
	filtersScrollView:SetBackgroundColor(0, 0, 0, 0)
	filtersScrollView:SetPoint("TOPLEFT", uiWin.filterTextfield, "BOTTOMLEFT", 1, 10)
	filtersScrollView:SetPoint("RIGHT", uiWin.filterTextfield, "RIGHT", -1, nil)
	filtersScrollView:SetPoint("BOTTOM", uiWin.filtersTab, "BOTTOM", nil, -10)
	filtersScrollView:SetBorder(2, BORDER_R, BORDER_G, BORDER_B, 1)

	uiWin.filtersList = Library.LibSimpleWidgets.List(uiWin:GetName() .. ".filtersList", uiWin.filtersTab)
	uiWin.filtersList:SetBackgroundColor(0, 0, 0, 0)
	uiWin.filtersList:SetFontSize(UI_FONT_SIZE)
	uiWin.filtersList.Event.SelectionChange = KBBUI.OnFilterSelect
	filtersScrollView:SetContent(uiWin.filtersList)


	----------------------------------------------------------------------
	---------------------------- look tab --------------------------------

	uiWin.buffColorGroup = KBBUI.MakeColorSliderGroup(uiWin:GetName() .. ".buffColorGroup", uiWin.lookTab, 155, KBBLS("UI_BUFF_COLOR_LABEL_TEXT"))
	uiWin.buffColorGroup:SetPoint("TOPLEFT", uiWin.lookTab, "TOPLEFT", 10, 10)
	uiWin.buffColorGroup.rSlider.Event.SliderChange = KBBUI.OnBuffColorSliderChange
	uiWin.buffColorGroup.gSlider.Event.SliderChange = KBBUI.OnBuffColorSliderChange
	uiWin.buffColorGroup.bSlider.Event.SliderChange = KBBUI.OnBuffColorSliderChange

	uiWin.debuffColorGroup = KBBUI.MakeColorSliderGroup(uiWin:GetName() .. ".buffColorGroup", uiWin.lookTab, 155, KBBLS("UI_DEBUFF_COLOR_LABEL_TEXT"))
	uiWin.debuffColorGroup:SetPoint("TOPLEFT", uiWin.buffColorGroup, "BOTTOMLEFT", 0, 10)
	uiWin.debuffColorGroup.rSlider.Event.SliderChange = KBBUI.OnDebuffColorSliderChange
	uiWin.debuffColorGroup.gSlider.Event.SliderChange = KBBUI.OnDebuffColorSliderChange
	uiWin.debuffColorGroup.bSlider.Event.SliderChange = KBBUI.OnDebuffColorSliderChange

	uiWin.bgColorGroup = KBBUI.MakeColorSliderGroup(uiWin:GetName() .. ".bgColorGroup", uiWin.lookTab, 155, KBBLS("UI_BG_COLOR_LABEL_TEXT"))
	uiWin.bgColorGroup:SetPoint("TOPLEFT", uiWin.buffColorGroup, "TOPRIGHT", 10, 0)
	uiWin.bgColorGroup.rSlider.Event.SliderChange = KBBUI.OnBGColorSliderChange
	uiWin.bgColorGroup.gSlider.Event.SliderChange = KBBUI.OnBGColorSliderChange
	uiWin.bgColorGroup.bSlider.Event.SliderChange = KBBUI.OnBGColorSliderChange

	uiWin.bgColorCheckbox = KBBUI.MakeCheckbox(uiWin:GetName() .. ".bgColorCheckbox", uiWin.lookTab, KBBLS("UI_BG_COLOR_CHECKBOX_TEXT"))
	uiWin.bgColorCheckbox:SetPoint("TOPLEFT", uiWin.bgColorGroup, "BOTTOMLEFT", 0, 10)
	uiWin.bgColorCheckbox.Event.CheckboxChange = KBBUI.OnBGColorCheckboxChange

	local bgaLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".bgaLabel", uiWin.lookTab, KBBLS("UI_BAR_BG_OPACITY_LABEL_TEXT"))
	bgaLabel:SetPoint("TOPLEFT", uiWin.bgColorCheckbox, "BOTTOMLEFT", 0, 10)

	uiWin.bgaSlider = KBBUI.MakeSlider(uiWin:GetName() .. ".bgaCBSlider", uiWin.lookTab, 0, RGB_RANGE, 160, math.floor(KBB.defaults.group.backgroundAlpha))
	uiWin.bgaSlider:SetPoint("TOPLEFT", bgaLabel, "BOTTOMLEFT", 0, 10)
	uiWin.bgaSlider.Event.SliderChange = KBBUI.OnBGASliderChange

	uiWin.fontColorGroup = KBBUI.MakeColorSliderGroup(uiWin:GetName() .. ".fontColorGroup", uiWin.lookTab, 155, KBBLS("UI_FONT_COLOR_LABEL_TEXT"))
	uiWin.fontColorGroup:SetPoint("TOPLEFT", uiWin.bgColorGroup, "TOPRIGHT", 10, 0)
	uiWin.fontColorGroup.rSlider.Event.SliderChange = KBBUI.OnFontColorSliderChange
	uiWin.fontColorGroup.gSlider.Event.SliderChange = KBBUI.OnFontColorSliderChange
	uiWin.fontColorGroup.bSlider.Event.SliderChange = KBBUI.OnFontColorSliderChange

	uiWin.debuffFontColorGroup = KBBUI.MakeColorSliderGroup(uiWin:GetName() .. ".debuffFontColorGroup", uiWin.lookTab, 155, KBBLS("UI_DEBUFF_FONT_COLOR_LABEL_TEXT"))
	uiWin.debuffFontColorGroup:SetPoint("TOPLEFT", uiWin.fontColorGroup, "BOTTOMLEFT", 0, 10)
	uiWin.debuffFontColorGroup.rSlider.Event.SliderChange = KBBUI.OnDebuffFontColorSliderChange
	uiWin.debuffFontColorGroup.gSlider.Event.SliderChange = KBBUI.OnDebuffFontColorSliderChange
	uiWin.debuffFontColorGroup.bSlider.Event.SliderChange = KBBUI.OnDebuffFontColorSliderChange

	local fontSizeLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".fontSizeLabel", uiWin.lookTab, KBBLS("UI_BAR_FONT_SIZE_LABEL_TEXT"))
	fontSizeLabel:SetPoint("TOPLEFT", uiWin.debuffFontColorGroup, "BOTTOMLEFT", 0, 10)

	uiWin.fontSizeSlider = KBBUI.MakeSlider(uiWin:GetName() .. ".fontSizeSlider", uiWin.lookTab, MIN_FONT_SIZE, MAX_FONT_SIZE, 160, 14)
	uiWin.fontSizeSlider:SetPoint("TOPLEFT", fontSizeLabel, "BOTTOMLEFT", 0, 10)
	uiWin.fontSizeSlider.Event.SliderChange = KBBUI.OnFontSizeSliderChange

	uiWin.colorsPreviewWidget = KBBUI.MakeColorsPreviewWidget(uiWin:GetName() .. ".colorsPreviewWidget", uiWin.lookTab, KBBLS("UI_COLOR_PREVIEW_LABEL_TEXT"), 155, 20, KBBLS("UI_COLOR_PREVIEW_BAR_TEXT"),  UI_FONT_SIZE)
	uiWin.colorsPreviewWidget:SetPoint("TOPLEFT", uiWin.bgaSlider, "BOTTOMLEFT", 0, 20)


	--------------------- profiles tab -------------------------------------

	local currentProfileLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".currentProfileLabel", uiWin.profilesTab, KBBLS("UI_CURRENT_PROFILE_LABEL_TEXT"))
	currentProfileLabel:SetPoint("TOPLEFT", uiWin.profilesTab, "TOPLEFT", 30, 30)

	local exportProfileLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".exportProfileLabel", uiWin.profilesTab, KBBLS("UI_EXPORT_PROFILE_LABEL_TEXT"))
	exportProfileLabel:SetPoint("TOPLEFT", currentProfileLabel, "BOTTOMLEFT", 0, 10)

	uiWin.exportProfileTextfield = UI.CreateFrame("RiftTextfield", uiWin:GetName() .. ".exportProfileTextfield", uiWin.profilesTab)
	uiWin.exportProfileTextfield:SetPoint("BOTTOMLEFT", exportProfileLabel, "BOTTOMRIGHT", 10, 0)
	uiWin.exportProfileTextfield:SetBackgroundColor(0, 0, 0, EDITFIELD_ALPHA)
	uiWin.exportProfileTextfield:SetText("")

	uiWin.exportProfileConfirmButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".exportProfileConfirmButton", uiWin.profilesTab)
	uiWin.exportProfileConfirmButton:SetText(KBBLS("UI_EXPORT_PROFILE_BUTTON_TEXT"))
	uiWin.exportProfileConfirmButton:SetPoint("CENTERLEFT", uiWin.exportProfileTextfield, "CENTERRIGHT", 0, -2)
	uiWin.exportProfileConfirmButton.Event.LeftDown = KBBUI.OnExportProfileClick

	uiWin.currentProfileNameLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".currentProfileNameLabel", uiWin.profilesTab, "Test")
	uiWin.currentProfileNameLabel:SetPoint("BOTTOM", currentProfileLabel, "BOTTOM")
	uiWin.currentProfileNameLabel:SetPoint("LEFT", uiWin.exportProfileTextfield, "LEFT")
	uiWin.currentProfileNameLabel:SetPoint("RIGHT", uiWin.exportProfileTextfield, "RIGHT")
	uiWin.currentProfileNameLabel:SetFontColor(1, 1, 1, 1)

	local importProfileLabel = KBBUI.MakeLabel(uiWin:GetName() .. ".importProfileLabel", uiWin.profilesTab, KBBLS("UI_IMPORT_PROFILE_LABEL_TEXT"))
	importProfileLabel:SetPoint("TOPLEFT", exportProfileLabel, "BOTTOMLEFT", 0, 10)

	local profilesScrollView = Library.LibSimpleWidgets.ScrollView(uiWin:GetName() .. ".profilesScrollView", uiWin.profilesTab)
	profilesScrollView:SetBackgroundColor(0, 0, 0, 0)
	profilesScrollView:SetPoint("RIGHT", uiWin.exportProfileTextfield, "RIGHT", -1, nil)
	profilesScrollView:SetPoint("LEFT", uiWin.exportProfileTextfield, "LEFT", 1, nil)
	profilesScrollView:SetPoint("TOP", importProfileLabel, "BOTTOM", nil, -10)
	profilesScrollView:SetPoint("BOTTOM", uiWin.profilesTab, "BOTTOM", nil, -30)
	profilesScrollView:SetBorder(2, BORDER_R, BORDER_G, BORDER_B, 1)

	uiWin.profilesList = Library.LibSimpleWidgets.List(uiWin:GetName() .. ".profilesList", uiWin.profilesTab)
	uiWin.profilesList:SetBackgroundColor(0, 0, 0, 0)
	uiWin.profilesList:SetFontSize(16)
	profilesScrollView:SetContent(uiWin.profilesList)

	uiWin.importProfileButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".importProfileButton", uiWin.profilesTab)
	uiWin.importProfileButton:SetText(KBBLS("UI_IMPORT_PROFILE_BUTTON_TEXT"))
	uiWin.importProfileButton:SetPoint("TOPLEFT", profilesScrollView, "TOPRIGHT", 0, -2)
	uiWin.importProfileButton.Event.LeftDown = KBBUI.OnImportProfileClick

	uiWin.deleteProfileButton = UI.CreateFrame("RiftButton", uiWin:GetName() .. ".deleteProfileButton", uiWin.profilesTab)
	uiWin.deleteProfileButton:SetText(KBBLS("UI_DELETE_PROFILE_BUTTON_TEXT"))
	uiWin.deleteProfileButton:SetPoint("TOPLEFT", uiWin.importProfileButton, "BOTTOMLEFT", 0, 20)
	uiWin.deleteProfileButton.Event.LeftDown = KBBUI.OnDeleteProfileClick

	local separator2 = UI.CreateFrame("Frame", uiWin:GetName() .. ".separator2", uiWin.profilesTab)
	separator2:SetBackgroundColor(BORDER_R, BORDER_G, BORDER_B, 1)
	separator2:SetWidth(2)
	separator2:SetPoint("TOP", uiWin.profilesTab, "TOP", nil, 25)
	separator2:SetPoint("BOTTOM", uiWin.profilesTab, "BOTTOM", nil, -25)
	separator2:SetPoint("LEFT", uiWin.deleteProfileButton, "RIGHT", 10, nil)

	local warningTextFrame = UI.CreateFrame("Text", uiWin:GetName() .. ".warningTextFrame", uiWin.profilesTab)
	warningTextFrame:SetPoint("CENTERLEFT", separator2, "CENTERRIGHT", 15, 0)
	warningTextFrame:SetWordwrap(true)
	warningTextFrame:SetPoint("RIGHT", uiWin.profilesTab, "RIGHT", -30, nil)
	warningTextFrame:SetFontSize(18)
	warningTextFrame:SetFontColor(1, 1, 1, 1)
	warningTextFrame:SetText(KBBLS("UI_PROFILE_IMPORT_WARNING"))



	function uiWin:SelectGroupWithIndex(index)
		self.groupsList:SetSelectedIndex(index)
	end

	function uiWin:GetCurrentChar()
		if (not self.currentChar) then

			local shard = Inspect.Shard().name
			local character = Inspect.Unit.Detail("player").name
			local thisChar = character .. "@" .. shard

			self.currentChar = thisChar
		end

		return self.currentChar
	end
end

------------------------------------------------------------------------
KBBUI.SetGroupButtonsState =  function (flag)
	KBB.uiWindow.removeButton:SetEnabled(flag)
	KBB.uiWindow.renameButton:SetEnabled(flag)
end
KBBUI.LoadGroupList = function(profile)
	if not (KBB.uiWindow) then return end
	local db = KBB.KBB_Settings_.settings[profile].groups
	local names = {}
	local ids = {}
	for _, group in ipairs(db) do
		table.insert(names, group.name)
		table.insert(ids, group.id)
	end
	KBB.uiWindow.groupsList:SetItems(names, ids)
	local n = #(names);
    if (n ==0) then KBBUI.SetGroupButtonsState(false) end
	return n
end

KBBUI.LoadGroupUnit = function(group, uiWin)
	local unit = group.settings.unit
	if (unit == "player") then
		uiWin.unitRadios:SetSelectedIndex(1, true)
	elseif (unit == "player.target") then
		uiWin.unitRadios:SetSelectedIndex(2, true)
	elseif (unit == "focus") then
		uiWin.unitRadios:SetSelectedIndex(3, true)
	elseif (unit == "player.pet") then
		uiWin.unitRadios:SetSelectedIndex(4, true)
	end
end
KBBUI.LoadGroupWatchingType = function(group, uiWin)
	local watchType = group.settings.watchType
	if (watchType == "buff") then
		uiWin.watchTypeRadios:SetSelectedIndex(1, true)
	else
		uiWin.watchTypeRadios:SetSelectedIndex(2, true)
	end
end
KBBUI.LoadGroupFilterType = function(group, uiWin)
	local filterType = group.settings.filterType
	if (filterType == "whitelist") then
		uiWin.filterTypeRadios:SetSelectedIndex(1, true)
	else
		uiWin.filterTypeRadios:SetSelectedIndex(2, true)
	end
end
KBBUI.LoadGroupSortOrder = function (group, uiWin)
	local sorts = group.settings.sorts
	if (sorts) then
		local sortOrder = group.settings.sortOrder
		if (sortOrder == "asc") then
			uiWin.sortRadios:SetSelectedIndex(1, true)
		else
			uiWin.sortRadios:SetSelectedIndex(2, true)
		end
	else
		uiWin.sortRadios:SetSelectedIndex(3, true)
	end
end
KBBUI.LoadGroupGrowthDirection = function(group, uiWin)
	local growth = group.settings.growth
	if growth == "up" then
		uiWin.growthRadios:SetSelectedIndex(1, true)
	else
		uiWin.growthRadios:SetSelectedIndex(2, true)
	end
end
KBBUI.LoadGroupCancelBuff = function(group, uiWin)
	local cancelClick = group.settings.cancelClick
	uiWin.cancelCheckbox:SetChecked(cancelClick, true)
end
KBBUI.LoadGroupSize = function(group, uiWin)
	local barHeight = group.settings.barHeight
	local barWidth = group.settings.barWidth
	local xRange = math.max(barWidth, MAX_BAR_WIDTH)
	local yRange = math.max(barHeight, MAX_BAR_HEIGHT)
	uiWin.widthSlider:SetRange(10, xRange)
	uiWin.heightSlider:SetRange(10, yRange)
	uiWin.widthSlider:SetPosition(barWidth, true)
	uiWin.heightSlider:SetPosition(barHeight, true)
end
KBBUI.LoadGroupPosition = function(group, uiWin)
	local x = group.settings.position.x
	local y = group.settings.position.y
	if not KBB.screenSize.w then KBB.screenSize.w = UIParent:GetWidth() end
	if not KBB.screenSize.h then KBB.screenSize.h = UIParent:GetHeight() end
	local xRange = math.floor(math.max(KBB.screenSize.w/ 2, x))
	local yRange = math.floor(math.max(KBB.screenSize.h / 2, y))
	uiWin.xSlider.nochange = true
	uiWin.ySlider.nochange = true
 	uiWin.xSlider:SetRange(-xRange, xRange)
 	uiWin.ySlider:SetRange(-yRange, yRange)
	uiWin.xSlider:SetPosition(x, true)
	uiWin.ySlider:SetPosition(y, true)
	uiWin.xSlider.nochange = false
	uiWin.ySlider.nochange = false
end
KBBUI.LoadGroupFilters = function(group, uiWin)
	local filters = group.settings.filters

	local strings = {}
	local ids = {}
	for k, v in ipairs(filters) do
		table.insert(strings, v.filterString)
		table.insert(ids, k)
	end
	uiWin.filtersList:SetItems(strings, ids)
	KBBUI.SetFilterButtonsState()

	return #(ids)
end

KBBUI.LoadGroupColors = function(group, uiWin)
	local buffColor = group.settings.buffColor
	local debuffColor = group.settings.debuffColor
	local alpha = group.settings.backgroundAlpha
	local fontColor = group.settings.fontColor
	local debuffFontColor = group.settings.debuffFontColor
	local bgUsesFgColor = group.settings.backgroundUsesSameColor
	local bgColor = group.settings.backgroundColor

	-- I have separate settings for text, timer and stack, maybe I'll add separate controls later
	local fontSize = group.settings.textFontSize
	local mn, mx = uiWin.fontSizeSlider:GetRange()
	if fontSize < mn then mn = fontSize end
	if fontSize > mx then mx = fontSize end
	uiWin.fontSizeSlider:SetRange(mn, mx)
	uiWin.fontSizeSlider:SetPosition(fontSize, true)

	uiWin.buffColorGroup.rSlider:SetPosition(math.floor(buffColor.r ), true)
	uiWin.buffColorGroup.gSlider:SetPosition(math.floor(buffColor.g ), true)
	uiWin.buffColorGroup.bSlider:SetPosition(math.floor(buffColor.b ), true)

	uiWin.debuffColorGroup.rSlider:SetPosition(math.floor(debuffColor.r), true)
	uiWin.debuffColorGroup.gSlider:SetPosition(math.floor(debuffColor.g), true)
	uiWin.debuffColorGroup.bSlider:SetPosition(math.floor(debuffColor.b), true)

	uiWin.bgColorGroup.rSlider:SetPosition(math.floor(bgColor.r), true)
	uiWin.bgColorGroup.gSlider:SetPosition(math.floor(bgColor.g), true)
	uiWin.bgColorGroup.bSlider:SetPosition(math.floor(bgColor.b), true)

	uiWin.fontColorGroup.rSlider:SetPosition(math.floor(fontColor.r), true)
	uiWin.fontColorGroup.gSlider:SetPosition(math.floor(fontColor.g), true)
	uiWin.fontColorGroup.bSlider:SetPosition(math.floor(fontColor.b), true)

	uiWin.debuffFontColorGroup.rSlider:SetPosition(math.floor(debuffFontColor.r), true)
	uiWin.debuffFontColorGroup.gSlider:SetPosition(math.floor(debuffFontColor.g), true)
	uiWin.debuffFontColorGroup.bSlider:SetPosition(math.floor(debuffFontColor.b), true)

	uiWin.bgaSlider:SetPosition(math.floor(alpha), true)

	uiWin.bgColorCheckbox.nochange = true
	uiWin.bgColorCheckbox:SetChecked(bgUsesFgColor, true)
	uiWin.bgColorCheckbox.nochange = false
	uiWin.bgColorGroup:SetEnabled(not bgUsesFgColor)

	KBBUI.UpdateBuffColorPreview(group, uiWin)
	KBBUI.UpdateDebuffColorPreview(group, uiWin)
	KBBUI.UpdateFontColorPreview(group, uiWin)
end

KBBUI.LoadGroupDetails = function(group, uiWin)
	KBB.uiWindow.filterInputArea:SetVisible(false)
	KBBUI.LoadGroupUnit(group, uiWin)
	KBBUI.LoadGroupWatchingType(group, uiWin)
	KBBUI.LoadGroupFilterType(group, uiWin)
	KBBUI.LoadGroupSortOrder(group, uiWin)
	KBBUI.LoadGroupGrowthDirection(group, uiWin)
	 KBBUI.LoadGroupCancelBuff(group, uiWin)
	KBBUI.LoadGroupSize(group, uiWin)
	KBBUI.LoadGroupPosition(group, uiWin)
	KBBUI.LoadGroupFilters(group, uiWin)
	KBBUI.LoadGroupColors(group, uiWin)
end

KBBUI.LoadProfilesList = function()
--	if not (KBB.uiWindow) then return end
	local db = KBB.KBB_Settings_.settings
	local names = {}
	local thisChar = KBB.uiWindow:GetCurrentChar()
	for k, _ in pairs(db) do
		if (k ~= thisChar) then table.insert(names, k) end
	end
	KBB.uiWindow.currentProfileNameLabel:SetText(thisChar)
	KBB.uiWindow.currentProfileNameLabel.KLHshadow:SetText(thisChar)
	KBB.uiWindow.profilesList:SetItems(names)
	return names
end

KBBUI.UpdateBuffColorPreview = function(group, uiWin)
	local buffColor = group.settings.buffColor
	local alpha = group.settings.backgroundAlpha
	local bgUsesFgColor = group.settings.backgroundUsesSameColor
	local bgColor = group.settings.backgroundColor

	uiWin.colorsPreviewWidget.buffBarFg:SetBackgroundColor(buffColor.r / 255, buffColor.g / 255, buffColor.b / 255)

	if (bgUsesFgColor) then
		uiWin.colorsPreviewWidget.buffBarBg:SetBackgroundColor(buffColor.r / 255, buffColor.g / 255, buffColor.b / 255, alpha / 255)
	else
		uiWin.colorsPreviewWidget.buffBarBg:SetBackgroundColor(bgColor.r / 255, bgColor.g / 255, bgColor.b / 255, alpha / 255)
	end
end
KBBUI.UpdateDebuffColorPreview = function(group, uiWin)
	local debuffColor = group.settings.debuffColor
	local alpha = group.settings.backgroundAlpha
	local bgUsesFgColor = group.settings.backgroundUsesSameColor
	local bgColor = group.settings.backgroundColor

	uiWin.colorsPreviewWidget.debuffBarFg:SetBackgroundColor(debuffColor.r / 255, debuffColor.g / 255, debuffColor.b / 255)

	if (bgUsesFgColor) then
		uiWin.colorsPreviewWidget.debuffBarBg:SetBackgroundColor(debuffColor.r / 255, debuffColor.g / 255, debuffColor.b / 255, alpha / 255)
	else
		uiWin.colorsPreviewWidget.debuffBarBg:SetBackgroundColor(bgColor.r / 255, bgColor.g / 255, bgColor.b / 255, alpha / 255)
	end
end
KBBUI.UpdateFontColorPreview = function(group, uiWin)
	local fontColor = group.settings.fontColor
	local debuffFontColor = group.settings.debuffFontColor
	uiWin.colorsPreviewWidget.buffText:SetFontColor(fontColor.r/255, fontColor.g/255, fontColor.b / 255)
	uiWin.colorsPreviewWidget.debuffText:SetFontColor(debuffFontColor.r / 255, debuffFontColor.g / 255, debuffFontColor.b / 255)
end

KBBUI.UpdateBGColorPreview = function (group, uiWin)
	local alpha = group.settings.backgroundAlpha
	local bgUsesFgColor = group.settings.backgroundUsesSameColor

	if (bgUsesFgColor) then
		local buffColor = group.settings.buffColor
		local debuffColor = group.settings.debuffColor
		uiWin.colorsPreviewWidget.debuffBarBg:SetBackgroundColor(debuffColor.r / 255, debuffColor.g / 255, debuffColor.b / 255, alpha / 255)
		uiWin.colorsPreviewWidget.buffBarBg:SetBackgroundColor(buffColor.r / 255, buffColor.g / 255, buffColor.b / 255, alpha / 255)
	else
		local bgColor = group.settings.backgroundColor
		uiWin.colorsPreviewWidget.buffBarBg:SetBackgroundColor(bgColor.r / 255, bgColor.g / 255, bgColor.b / 255, alpha / 255)
		uiWin.colorsPreviewWidget.debuffBarBg:SetBackgroundColor(bgColor.r / 255, bgColor.g / 255, bgColor.b / 255, alpha / 255)
	end

end

KBBUI.SetFilterButtonsState = function()
	local uiWin = KBB.uiWindow
	local selected = (uiWin.filtersList:GetSelectedIndex() ~= nil)
	uiWin.removeFilterButton:SetEnabled(selected)
	uiWin.editFilterButton:SetEnabled(selected)
end

KBBUI.OnGroupSelect = function(list)
	local groupID = list:GetSelectedValue()
	if groupID then
		Library.KLH.makeChildrenPerformSelector(KBB.uiWindow.groupsTab, true, "SetEnabled", true)
		KBBUI.SetGroupButtonsState(true)
	else
		Library.KLH.makeChildrenPerformSelector(KBB.uiWindow.groupsTab, true, "SetEnabled", false)
		KBBUI.SetGroupButtonsState(false)
	end
	local uiWin = KBB.uiWindow                                                                               
	local _, group = KBB.GroupWithId(groupID)
	if not group then return end
	KBBUI.LoadGroupDetails(group, uiWin)

end


------------------------------------------------------------------------
local function uiLoadGroups()
	local uiWin = KBB.uiWindow
--	if (not uiWin) then return end

	KBBUI.LoadGroupList(uiWin:GetCurrentChar())
end
local function uiLoadSettings()
	local uiWin = KBB.uiWindow
--	if (not uiWin) then return end

	local settings = KBB.characterDB.settings
	uiWin.minimapCheckbox.nochange = true
	uiWin.minimapCheckbox:SetChecked(settings.minimapButton)
	uiWin.minimapCheckbox.nochange = false
	uiWin.minimapSlider:SetEnabled(settings.minimapButton)
	uiWin.minimapSlider:SetPosition(settings.minimapButtonAngle, true)
	uiWin.minimapOffsetSlider:SetEnabled(settings.minimapButton)
	uiWin.minimapOffsetSlider:SetPosition(settings.minimapButtonOffset, true)
	uiWin.minimapButtonSizeSlider:SetEnabled(settings.minimapButton)
	uiWin.minimapButtonSizeSlider:SetPosition(settings.minimapButtonSize, true)
	uiWin.nativeTooltipCheckbox.nochange = true
	uiWin.nativeTooltipCheckbox:SetChecked(settings.useNativeTooltip)
	uiWin.nativeTooltipCheckbox.nochange = false

	uiWin.enabledCheckbox.nochange = true
	uiWin.enabledCheckbox:SetChecked(settings.on)
	uiWin.enabledCheckbox.nochange = false
end
local function uiLoadProfiles()
	KBBUI.LoadProfilesList()
end
---------------------- UI toggle -----------------------------
function KBB.ToggleUI()
-- 	print("toggleUI")
	if (not KBB.uiWindow) then KBB.MakeUI() end
	local show = not KBB.uiWindow:GetVisible()
	if (show) then
		
		uiLoadGroups()
		uiLoadProfiles()
		uiLoadSettings()
		KBB.uiWindow:SelectGroupWithIndex(1)
		--		SetElementsState()
	else
		KBBUI.OnClose()
	end

	KBB.uiWindow:SetVisible(show)
	KBB.mmbContext:SetLayer(KBB.uiContext:GetLayer() - 1)
end

------ minimap size changes
local mapMiniSizeChange = UI.Native.MapMini.Event.Size

UI.Native.MapMini.Event.Size = function(...)
	if mapMiniSizeChange then mapMiniSizeChange(...) end
	if not UI.Native.MapMini:GetLoaded() then return end

	if (KBB.characterDB) then
		local settings = KBB.characterDB.settings
		if (settings) then   
			Library.KLH.attachButtonToMinimap(KBB.minimapButton, settings.minimapButtonAngle, settings.minimapButtonOffset)
		end
	end
end

------ minimap visibility changes
-- local mapMiniLoadedChange = UI.Native.MapMini.Event.Loaded
-- UI.Native.MapMini.Event.Loaded = function(...)
-- 	if mapMiniLoadedChange then mapMiniLoadedChange(...) end
-- 	print("UI.Native.MapMini.Event.Loaded", ..., UI.Native.MapMini:GetLoaded())
-- end


-- local mapMiniLayerChange = UI.Native.MapMini.Event.Layer
-- UI.Native.MapMini.Event.Layer = function(...)
-- 	if mapMiniLayerChange then mapMiniLayerChange(...) end
-- 	print("UI.Native.MapMini.Event.Layer", ..., UI.Native.MapMini:GetLayer())
-- 	KBB.minimapButton:SetLayer(UI.Native.MapMini:GetLayer())
-- end
-- 
-- local mapMiniStrataChange = UI.Native.MapMini.Event.Strata
-- UI.Native.MapMini.Event.Strata = function(...)
-- 	if mapMiniStrataChange then mapMiniStrataChange(...) end
-- 	print("UI.Native.MapMini.Event.Strata", ..., UI.Native.MapMini:GetStrata())
-- 	KBB.minimapButton:SetStrata(UI.Native.MapMini:GetStrata()())
-- end

------- screen size changes
local uiparentSizeChange = UIParent.Event.Size
UIParent.Event.Size = function(...)
	if uiparentSizeChange then uiparentSizeChange(...) end

	KBB.screenSize.w = UIParent:GetWidth()
	KBB.screenSize.h = UIParent:GetHeight()
end
