if not Library then Library = {} end
if not Library.KLH then Library.KLH = {} end

Library.KLH.escape = function(s)
  return (s:gsub('[%-%.%+%[%]%(%)%$%^%%%?%*]','%%%1'):gsub('%z','%%z'))
end

Library.KLH.strsplit = function  (str, delimiter)
  assert (type (delimiter) == "string" and string.len (delimiter) > 0,
          "bad delimiter")  
	-- http://lua-users.org/wiki/SplitJoin
	local t = {}  -- NOTE: use {n = 0} in Lua-5.0
	local d = Library.KLH.escape(delimiter)
	local fpat = "(.-)" .. d
	local last_end = 1
	local s, e, cap = str:find(fpat, 1)
	while s do
		if s ~= 1 or cap ~= "" then
			table.insert(t,cap)
		end
		last_end = e+1
		s, e, cap = str:find(fpat, last_end)
	end
	if last_end <= #str then
		cap = str:sub(last_end)
		table.insert(t, cap)
	end
	return t

end -- function split

Library.KLH.trim = function (s) -- trim leading and trailing whitespaces
  -- from PiL2 20.4
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end



Library.KLH.tableContains = function (table, element)
  for _, value in pairs(table) do
    if value == element then
      return true
    end
  end
  return false
end

Library.KLH.tdump = function (t, brackets, prefix)	
	local lookup_table = {}
	local function tdump_(t, brackets, prefix)
		prefix = prefix or ""
		brackets = brackets or false
		if (type(t) == "table") then
			local key
			lookup_table[t] = true
			for k, v in pairs(t) do 
				key = ""
				if (type(k) == "number") then
					if (brackets) then key = "["..tostring(k).."] = " end
				else
					key = tostring(k).." = " 
				end
				if (type(v) == "table") then
					if lookup_table[v] then -- protection against recursion
						print(prefix..key..tostring(v))
						break
					end
					print(prefix..key.."{")
					tdump_(v, brackets, prefix.."  ")
					print(prefix.."}")
				else
				    local quotes = ""
					if type(v) == "string" then quotes = "\"" end
					print(prefix..key..quotes..tostring(v)..quotes)
				end
			end 
			lookup_table[t] = nil
		else
			print(prefix..tostring(t))
		end
	end
 	
 	tdump_(t, brackets, prefix)
end -- function tdump

Library.KLH.tstring = function (t, noLineBreaks, brackets, prefix)	
	local lookup_table = {}
	local function tdump_(t, noLineBreaks, brackets, prefix)
        local prefix = prefix or ""
--        local result = ""
		local result_table = {}
		local brackets = brackets or false
		local lineBreak, lineBreakBracket
		if noLineBreaks then 
			lineBreak = ", " lineBreakBracket = "" prefix = ""
		else 
			lineBreak = ",\n" 
			lineBreakBracket = "\n"
		end
		if (type(t) == "table") then
			local key
			lookup_table[t] = true
			local tmp_t
			prefix = prefix.."  "
			for k, v in pairs(t) do 
				key = ""
				if (type(k) == "number") then
					if (brackets) then key = "["..tostring(k).."] = " end
				else
-- 					key = tostring(k).." = " 
					key = "[\"".. tostring(k).."\"]" .." = " 
				end
				if (type(v) == "table") then
					if lookup_table[v] then
						table.insert(result_table, prefix)
						table.insert(result_table, key)
						table.insert(result_table, tostring(v))
						table.insert(result_table, lineBreak)
					else
						table.insert(result_table, prefix)
						table.insert(result_table, key)
						table.insert(result_table, "{")
						table.insert(result_table, lineBreakBracket)
						tmp_t = tdump_(v, noLineBreaks, brackets, prefix .. "  ")
						if (type(tmp_t) == "table") then
							table.insert(result_table, table.concat(tmp_t) )
						else
							table.insert(result_table, tmp_t)
						end
						table.insert(result_table, prefix)
						table.insert(result_table, "}")
						table.insert(result_table, lineBreak)
					end
				else
					table.insert(result_table, prefix)
					table.insert(result_table, key)
					local quotes = ""
					if type(v) == "string" then 
						quotes = "\""
						v = string.gsub(v, "\n", "\\n") 
					end
					table.insert(result_table, quotes)
					table.insert(result_table, tostring(v))
					table.insert(result_table, quotes)
					table.insert(result_table, lineBreak)
				end
			end 
			lookup_table[t] = nil
		else
			  table.insert(result_table, prefix)
			  table.insert(result_table, key)
			  table.insert(result_table, tostring(t))
			  table.insert(result_table, lineBreak)
 		end
		return table.concat(result_table)
 	end
 	
 	local result = tdump_(t, noLineBreaks, brackets, prefix)
 --	if string.sub(result, -1) == "\n" then  result = string.sub(result, 1, -2) end
 	local lineBreak, lineBreakBracket
		if noLineBreaks then 
			lineBreak = ", " lineBreakBracket = "" prefix = ""
		else 
			lineBreak = "\n" 
			lineBreakBracket = "\n"
		end
 	result = "{"..lineBreakBracket..result.."}"
 	return result
end -- function tstring

Library.KLH.unregisterEventHandler = function (event, addonId, handlerId)
--[[	local handlers = {}
	for i, v in pairs(event) do
		if v[2] == addonId and v[3] == handlerId then
			table.insert(handlers, i)
		end
	end
	for _, k in pairs(handlers) do
		event[k] = nil
	end
]]--
	-- event table is iterated with ipairs. I have to hope that there isn't more than one handler with given (addonId, handlerId) pair
	for i, v in ipairs(event) do
		if v[2] == addonId and v[3] == handlerId then
			table.remove(event, i)
			break
		end
	end
end -- function UnregisterEventHandler



Library.KLH.timeString = function (timeInSeconds, tenths)
	local formatString = ""
	local hh = 0
	local mm = 0
	local ss = 0
	if (timeInSeconds < 60) then
		ss = timeInSeconds
		formatString = "ss"
	elseif (timeInSeconds < 3600) then
		mm = math.floor(timeInSeconds/60)
		ss = timeInSeconds - mm*60
		formatString = "mm:ss"
	else
		hh = math.floor(timeInSeconds/3600)
		timeInSeconds = timeInSeconds - 3600*hh
		mm = math.floor(timeInSeconds/60)
		ss = timeInSeconds - mm*60
		formatString = "hh:mm:ss"
	end
	local result = ""
	local _, fr = math.modf(ss)
	if (tenths and math.abs(fr) >= 0.1) then result = string.format("%03.1f", ss)
	else
		result = string.format("%02d", math.floor(ss))
	end
	result = string.format("%02d", mm)..":"..result
	if (hh ~=0 ) then result = string.format("%02d", hh)..":"..result end
	return result
end


-- http://lua-users.org/wiki/CopyTable
Library.KLH.deepcopy = function (object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

Library.KLH.makeWindowMovable = function (window, anchorFrame)

	anchorFrame.dragWindow = window
	
	function anchorFrame.Event:LeftDown()
		self.leftDown = true
		self.dragData = {}
		local mouseData = Inspect.Mouse()
		
		self.dragData.dX = self.dragWindow:GetLeft() - mouseData.x
		self.dragData.dY = self.dragWindow:GetTop() - mouseData.y
	end
	
	function anchorFrame.Event:LeftUp()
	   self.leftDown = false
	
	end
	
	function anchorFrame.Event:MouseMove(x, y)
	   if not self.leftDown then return end
	   
	   local w = self.dragWindow:GetWidth()
	   local h = self.dragWindow:GetHeight()
	   self.dragWindow:ClearAll()    -- a kingdom for a :ClearAllPoints()
	   self.dragWindow:SetWidth(w)
	   self.dragWindow:SetHeight(h)
	   self.dragWindow:SetPoint("TOPLEFT", self.dragWindow:GetParent(), "TOPLEFT",
			x + self.dragData.dX,
			y + self.dragData.dY) 
	   
	end

end

local function dragLeftDown(frame)
	frame.KLHSetDraggable.isLeftDown = true
	local mouse = Inspect.Mouse()
	frame.KLHSetDraggable.xDiff = mouse.x - frame:GetLeft()
    frame.KLHSetDraggable.yDiff = mouse.y - frame:GetTop()
    local left, top = frame:GetBounds()
    local w, h = frame:GetWidth(), frame:GetHeight()
    frame:ClearAll()
    frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", left, top)
    frame:SetWidth(w)
    frame:SetHeight(h)
end
local function dragLeftUp(frame)
	frame.KLHSetDraggable.isLeftDown = false
end
local function dragLeftUpoutside(frame)
    frame.KLHSetDraggable.isLeftDown = false
end
local function dragMouseMove(frame, x ,y)
	if not frame.KLHSetDraggable.isLeftDown then return end
	frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x - frame.KLHSetDraggable.xDiff, y - frame.KLHSetDraggable.yDiff)
end

Library.KLH.setDraggable  = function (frame, draggable)
	if not frame.KLHSetDraggable then frame.KLHSetDraggable = {} end
	if draggable and not frame.shouldDrag then	
		frame.KLHSetDraggable.LeftDown = dragLeftDown
		frame.KLHSetDraggable.LeftUp = dragLeftUp
		frame.KLHSetDraggable.MouseMove = dragMouseMove
		frame.KLHSetDraggable.LeftUpoutside = dragLeftUpoutside
		
		frame.Event.LeftDown = frame.KLHSetDraggable.LeftDown
		frame.Event.LeftUp = frame.KLHSetDraggable.LeftUp
		frame.Event.MouseMove = frame.KLHSetDraggable.MouseMove
		frame.Event.LeftUpoutside = frame.KLHSetDraggable.LeftUpoutside
	elseif not draggable and frame.shouldDrag then
		frame.Event.LeftDown = frame.KLHSetDraggable.LeftDown
		frame.Event.LeftUp = frame.KLHSetDraggable.LeftUp
		frame.Event.MouseMove = frame.KLHSetDraggable.MouseMove
		frame.Event.LeftUpoutside = frame.KLHSetDraggable.LeftUpoutside
		
		frame.KLHSetDraggable.LeftDown = nil
		frame.KLHSetDraggable.LeftUp = nil
		frame.KLHSetDraggable.MouseMove = nil
		frame.KLHSetDraggable.LeftUpoutside = nil
	end 
	frame.shouldDrag = draggable 
end 


Library.KLH.addCloseButton = function (window, func)
	local closeButton = UI.CreateFrame("RiftButton", window:GetName().."_closeButton", window)
	closeButton:SetSkin("close")
   	closeButton:SetPoint("BOTTOMRIGHT", window:GetContent(), "TOPRIGHT", 
				closeButton:GetWidth() / 6 ,
 				- closeButton:GetHeight() / 6)
	if func then
		closeButton.Event.LeftPress = function(self)
			func()
			window:SetVisible(false)
		end
	else
 		closeButton.Event.LeftPress = function(self)
 			window:SetVisible(false)
		end
 	end
 	return closeButton
end

Library.KLH.addTopLeftButton = function(window, textureSource, texturePath)
	local closeButton = UI.CreateFrame("Frame", window:GetName().."_topLeftButton", window)
	
	local textureFrame =  UI.CreateFrame("Texture", window:GetName().."_topLeftButtonTexture", closeButton)
	textureFrame:SetTexture(textureSource, texturePath)
	textureFrame:SetAllPoints(closeButton)
	closeButton:SetWidth(36)
	closeButton:SetHeight(36)
	
	closeButton:SetPoint("BOTTOMLEFT", window:GetContent(), "TOPLEFT", 
				- closeButton:GetWidth() / 6 ,
 				- closeButton:GetHeight() / 6)
 	return closeButton
end

Library.KLH.getFirstNil = function(t)
	if type(t) ~= "table" then return nil end

	local n = 1
	while t[n] do
		n = n + 1
	end
	
	return n	
	
end

Library.KLH.makeShadow = function (src, r, g, b, a, offX, offY)
   local shadow = UI.CreateFrame("Text", src:GetName()..".shadow", src:GetParent())
   
   shadow:SetText(src:GetText())
   shadow:SetFontSize(src:GetFontSize())
   shadow:SetFont(src:GetFont())
   shadow:SetWordwrap(src:GetWordwrap()) 
   
   shadow:SetFontColor(r, g, b, a)
--   shadow:SetPoint("CENTER", src, "CENTER", offX, offY)
   shadow:SetPoint("TOPLEFT", src, "TOPLEFT", offX, offY)
   shadow:SetLayer(src:GetLayer() - 1)
   return shadow
end

Library.KLH.makeChildrenPerformSelector = function (obj, recursive, selectorName, ...)
	if not obj["GetChildren"] then return end

	local chTable = obj:GetChildren()
	if not chTable then return end

	for child, _ in pairs(chTable) do
		if child[selectorName] then
			child[selectorName](child, ...)

			if recursive then
				Library.KLH.makeChildrenPerformSelector (child, recursive, selectorName, ...)
			end

		end
	end

end

Library.KLH.makeObjectsPerformSelector = function (objs, selectorName, ...)
--	print("makeObjectsPerformSelector", selectorName, ...)
	if not objs then return end
	if type(objs) == "table" then
		for _, obj in pairs(objs) do
			if obj[selectorName] then 
--				print(obj:GetName(), selectorName, ...) 
				obj[selectorName](obj, ...) 
			end
		end
	else
		if objs[selectorName] then
--			print(obj:GetName(), selectorName, ...)  
			objs[selectorName](objs, ...) 
		end
	end
end

Library.KLH.versionIsOlder = function(old, new)
	if not old then return true
	elseif not new then return  false end
	local oe = Library.KLH.strsplit(old, ".")
	local ne = Library.KLH.strsplit(new, ".")
	
	local lt = true
	local maxN = math.max(#oe, #ne) 
	for  i =1, maxN do
		local o = tonumber(oe[i] or "0")
		local n = tonumber(ne[i] or "0")
		
		if o == n then
			if i == maxN then
				lt = false 
			end
		elseif o > n then
			lt = false
			break	
		end
	end
	
	return lt
end

Library.KLH.minimapButtonOffsets = function(angle, offset, bWidth, bHeight, mWidth, mHeight)
	local bWidth_2 = bWidth / 2
	local bHeight_2 = bHeight / 2
	local mWidth_2 = mWidth / 2
	local mHeight_2 = mHeight / 2

	local dx = 0
	local dy = 0      
	if angle == 0 or angle == 360 then
		dx = bWidth_2 + mWidth_2 + offset
	elseif angle == 90 then
		dy = -bHeight_2 - mHeight_2 - offset
	elseif angle == 180 then
		dx = -bWidth_2 - mWidth_2 - offset
	elseif angle == 270 then
		dy = bHeight_2 + mHeight_2 + offset
	
	elseif (315 <= angle and angle < 360) or (0 < angle and angle < 45) then
		dx = bWidth_2 + mWidth_2 + offset
		dy = (-dx) * math.tan(math.rad(angle))   
	elseif (45 <= angle and angle < 90) or (90 < angle and angle <= 135) then
		dy = -bHeight_2 - mHeight_2 - offset
		dx = (-dy) / math.tan(math.rad(angle)) 
	elseif (135 < angle and angle < 180) or (180 < angle and angle < 225) then
		dx = -bWidth_2 - mWidth_2 - offset
		dy = (-dx) * math.tan(math.rad(angle)) 
	elseif (225 <= angle and angle < 270) or (270 < angle and angle < 315) then
		dy = bHeight_2 + mHeight_2 + offset
		dx = (-dy) / math.tan(math.rad(angle)) 
	end   

	return dx, dy
end

Library.KLH.attachButtonToMinimap = function(button, angle, offset)
	local mmFrame = UI.Native.MapMini
	local mmWidth = mmFrame:GetWidth()
	local mmHeight = mmFrame:GetHeight()
--	print("minimap size: ", mmWidth, mmHeight)
	local bHeight = button:GetHeight()
	local bWidth = button:GetWidth()


	local dx, dy = Library.KLH.minimapButtonOffsets(angle, offset, bWidth, bHeight, mmWidth, mmHeight)
--	print("attaching button: angle " .. angle .. ", offset " .. offset .. ",dx/dy " .. dx .. "/" .. dy)

	button:SetPoint("CENTERCENTER", mmFrame, "CENTERCENTER", dx, dy)


--	local parent = button:GetParent()
--	if (parent ~= mmFrame) then
--		button:SetParent(UI.Native.MapMini)
--	end

end

Library.KLH.rectContainsRect = function (rect1, rect2)
	return rect1.x < rect2.x and rect1.y < rect2.y
		and rect1.x + rect1.width > rect2.x + rect2.width
		and rect1.y + rect1.height > rect2.y + rect2.height
end
