---
-- Created by IntelliJ IDEA.
-- User: Kreiri
-- Date: 13.02.12
-- Time: 19:27
-- To change this template use File | Settings | File Templates.
--

-- Helper Functions

local MAX_MAX = 2147483647
local MIN_MIN = -2147483648

local function resizeCurrentForRange(current, max)
	local oldCurrent = current:GetText()
	current:SetText(tostring(max))
	current:SetWidth(current:GetFullWidth())
	current:SetText(oldCurrent)
end


-- Public Functions

local function SetBorder(self, width, r, g, b, a)
	Library.LibSimpleWidgets.SetBorder(self.current, width, r, g, b, a)
	Library.LibSimpleWidgets.SetBorder(self.dropdown, width, r, g, b, a)
end

local function SetBackgroundColor(self, r, g, b, a)
	self.current:SetBackgroundColor(r, g, b, a)
	self.dropdown:SetBackgroundColor(r, g, b, a)
end

local function GetDefaultHeight(self)
	return self.slider:GetDefaultHeight() - 2
end

local function GetDefaultWidth(self)
	return self.slider:GetDefaultWidth() + self.current:GetWidth()
end

local function GetEnabled(self)
	return self.enabled
end

local function SetEnabled(self, enabled)
	self.enabled = enabled
	if enabled then
		self.current:SetFontColor(1, 1, 1, 1)
	else
		self.current:SetFontColor(0.5, 0.5, 0.5, 1)
	end
	self.slider:SetEnabled(enabled)
end

local function GetRange(self)
	return self.slider:GetRange()
end

local function SetRange(self, min, max)
	self.slider:SetRange(min, max)
	resizeCurrentForRange(self.current, max)
end

local function GetPosition(self)
	return self.slider:GetPosition()
end

local function SetPosition(self, position)
	self.slider:SetPosition(position)
	self.current:SetText(tostring(position))
end

local function UpdateValueFromInput(inputField)
	local widget = inputField:GetParent()
	if not widget:GetEnabled() then return end

	local curText = inputField:GetText()
	if (not curText or curText:len() == 0) then return end

	local value = tonumber(curText)
	if (not value) then return end

	if (value < MIN_MIN or value > MAX_MAX) then return end

   	local minA, maxA = widget.slider:GetRange()
   	local curNum = tonumber(curText)
	if (minA > curNum)  then
		if (widget.allowOutRange) then
			widget:SetRange(curNum, maxA)
			resizeCurrentForRange(widget.current, maxA)
		else
			curNum = minA
			inputField:SetText(tostring(curNum))
		end
	elseif (curNum > maxA)  then
		if (widget.allowOutRange) then
			widget:SetRange(minA, curNum)
			resizeCurrentForRange(widget.current, maxA)
		else
			curNum = maxA
			inputField:SetText(tostring(curNum))
		end
	end

	widget.slider:SetPosition(curNum)
end

local function GetAllowOutRange(widget)
	return widget.allowOutRange
end

local function SetAllowOutRange(widget, value)
	assert(type(value) == "boolean", string.format("Incorrect value type: %s, expected boolean", type(value)))
	widget.allowOutRange = value
end

function Library.LibSimpleWidgets.InputSlider(name, parent)
	local widget = UI.CreateFrame("Frame", name, parent)

	widget.enabled = true

	widget.slider = UI.CreateFrame("RiftSlider", widget:GetName() .. "Slider", widget)
	widget.slider:SetPoint("TOPLEFT", widget, "TOPLEFT", 0, 6)

	widget.current = UI.CreateFrame("Text", widget:GetName() .. "Current", widget)
	widget.current:SetBackgroundColor(0, 0, 0, 1)
	widget.current:SetPoint("TOPRIGHT", widget, "TOPRIGHT")
	widget.current:SetPoint("BOTTOMRIGHT", widget, "BOTTOMRIGHT")
	widget.current:SetWidth(50)
	Library.LibSimpleWidgets.SetBorder(widget.current, 1, 0.5, 0.5, 0.5, 1)

	widget.slider:SetPoint("BOTTOMRIGHT", widget.current, "BOTTOMLEFT", -10, 0)

	widget.input = UI.CreateFrame("RiftTextfield", widget:GetName() .. "Input", widget)
	widget.input:SetBackgroundColor(0, 0, 0, 1)
	widget.input:SetPoint("TOPLEFT", widget.current, "TOPRIGHT", 5, 0)
	widget.input:SetPoint("BOTTOMLEFT", widget.current, "BOTTOMRIGHT", 5, 0)
	widget.input:SetWidth(50)
	widget.input:SetText("")

	widget.allowOutRange = true

	widget.input.Event.KeyUp = UpdateValueFromInput

	function widget.slider.Event:SliderChange()
		widget.current:SetText(tostring(widget.slider:GetPosition()))
		if widget.Event.SliderChange then
			widget.Event.SliderChange(widget)
		end
	end

	function widget.slider.Event:SliderGrab()
		if widget.Event.SliderGrab then
			widget.Event.SliderGrab(widget)
		end
	end

	function widget.slider.Event:SliderRelease()
		if widget.Event.SliderRelease then
			widget.Event.SliderRelease(widget)
		end
	end

	widget.SetBorder = SetBorder
	widget.SetBackgroundColor = SetBackgroundColor
	widget.GetDefaultHeight = GetDefaultHeight
	widget.GetDefaultWidth = GetDefaultWidth
	widget.GetEnabled = GetEnabled
	widget.SetEnabled = SetEnabled
	widget.GetRange = GetRange
	widget.SetRange = SetRange
	widget.GetPosition = GetPosition
	widget.SetPosition = SetPosition
	widget.SetAllowOutRange  = SetAllowOutRange
	widget.GetAllowOutRange = GetAllowOutRange

	Library.LibSimpleWidgets.EventProxy(widget, { "SliderChange", "SliderGrab", "SliderRelease"})

	local min, max = widget.slider:GetRange()
	resizeCurrentForRange(widget.current, max)

	widget:SetHeight(widget:GetDefaultHeight())
	widget:SetWidth(widget:GetDefaultWidth())

	widget.current:SetText(tostring(widget.slider:GetPosition()))

	return widget
end
